# Fields used in BG ModVar data structures

from SpectrumCommon.Const.BG.BGConstLink import *

# general fields for keeping ID fields consistent when relating records to each other and for use in result records

BG_F_BUDGET_ID          = 'budgetID'
BG_F_MOD_ID             = 'modID'
BG_F_IND_TYPE_ID        = 'indTypeID'
BG_F_IND_ID             = 'indID'
BG_F_BUDGET_CAT_ID      = 'budgetCatID'
BG_F_FUND_SOURCE_ID     = 'fundSourceID'
BG_F_FUND_FW_ID         = 'fundFrameworkID' 
BG_F_IS_AGGREGATE       = 'isAggregate' 
BG_F_STAFF_LEVEL_ID_HW  = 'staffLevelID'                
BG_F_COMPENS_TYPE_ID_HW = 'compensTypeID'  
BG_F_VALUE              = 'value'

BG_F_COSTING_SECT_HS_COSTING_ID         = 'costingSectionHSCostingID'         # string; 
BG_F_COSTING_SUBSECT_HS_COSTING_ID      = 'costingSubsectionHSCostingID'      # string; 
BG_F_ACT_HS_COSTING_ID                  = 'activityHSCostingID'               # string; 
BG_F_COSTING_SECT_HS_PROG_MGMT_ID       = 'costingSectionHSProgMgmtID'        # string; 
BG_F_COSTING_SUBSECT_HS_PROG_MGMT_ID    = 'costingSubsectionHSProgMgmtID'     # string; 
BG_F_ACT_HS_PROG_MGMT_ID                = 'activityHSProgMgmtID'              # string; 

BG_F_COSTING_SECT_GV_COSTING_ID         = 'costingSectionGVCostingID'         # string;
BG_F_COSTING_SUBSECT_GV_COSTING_ID      = 'costingSubsectionGVCostingID'      # string;
BG_F_ACT_GV_COSTING_ID                  = 'activityGVCostingID'               # string;
BG_F_COSTING_SECT_GV_PROG_MGMT_ID       = 'costingSectionGVProgMgmtID'        # string;
BG_F_COSTING_SUBSECT_GV_PROG_MGMT_ID    = 'costingSubsectionGVProgMgmtID'     # string;
BG_F_ACT_GV_PROG_MGMT_ID                = 'activityGVProgMgmtID'              # string;

BG_F_COSTING_SECT_HF_COSTING_ID         = 'costingSectionHFCostingID'         # string;
BG_F_COSTING_SUBSECT_HF_COSTING_ID      = 'costingSubsectionHFCostingID'      # string;
BG_F_ACT_HF_COSTING_ID                  = 'activityHFCostingID'               # string;
BG_F_COSTING_SECT_HF_PROG_MGMT_ID       = 'costingSectionHFProgMgmtID'        # string;
BG_F_COSTING_SUBSECT_HF_PROG_MGMT_ID    = 'costingSubsectionHFProgMgmtID'     # string;
BG_F_ACT_HF_PROG_MGMT_ID                = 'activityHFProgMgmtID'              # string;

# budget records

BG_BUDGET_ID                                           = BG_F_BUDGET_ID                      # string; Numeric mstID for defaults (still cast as string). GUID for custom budgets.
BG_BUDGET_NAME                                         = 'name'                              # string; Holds custom name or overridden default name.
BG_BUDGET_STR_CONST                                    = 'stringConstant'                    # string
BG_BUDGET_COST_INTERVS_DRUGS_SUPPLIES_IC               = 'costIntervsOrDrugsSuppliesIC'      # integer; 1 = interventions, 2 = drugs/supplies
BG_BUDGET_AGGR_DELIV_CHANS_IC                          = 'aggrDelivChansIC'                  # boolean; Are we costing IC indicators across all delivery channels combined or each individually?
BG_BUDGET_CUSTOM                                       = 'custom'                            # boolean
BG_BUDGET_ACTIVE                                       = 'active'                            # boolean
BG_BUDGET_FW_ID                                        = BG_F_FUND_FW_ID                     # string; Numeric mstID for defaults (still cast as string). GUID for custom funding frameworks.
BG_BUDGET_COST_LEVEL_PC_ID                             = 'levelPC'                           # integer; costing section = 1, subsection = 2, or activity level = 3
#BG_BUDGET_AGGR_PROG_MGMT_IND_HW                       = 'aggrProgMgmtIndHW'                 # boolean
BG_BUDGET_AGGR_INSERV_TRAIN_ALL_STAFF_HW               = 'aggrInservTrainAllStaffHW'         # boolean; Are we aggregating in-service training across all dynamic staff types? (Only applicable if Specific training details option chosen in HW)
BG_BUDGET_AGGR_PRESERV_TRAIN_ALL_STAFF_HW              = 'aggrPreservTrainAllStaffHW'        # boolean; Are we aggregating pre-service training across all dynamic staff types?
BG_BUDGET_AGGR_TOTAL_COMPENS_ALL_STAFF_HW              = 'aggrTotalCompensAllStaffHW'        # 2d list of boolean; For each staffing level and compensation type, are we aggregating total compensation across all dynamic staff types?
BG_BUDGET_AGGR_STAFF_INSERV_TRAIN_ALL_STAFF_LEV_HW     = 'aggrStaffInservTrainAllStaffLevHW' # 1d list of boolean; For each dynamic staff type, are we aggregating in-service training across all staffing levels? (Only applicable if Percent of salary approach option chosen in HW)
BG_BUDGET_AGGR_STAFF_LEV_INSERV_TRAIN_ALL_STAFF_HW     = 'aggrStaffLevInservTrainAllStaffHW' # 1d list of boolean; For each staffing level, are we aggregating in-service training across all dynamic staff types? (Only applicable if Percent of salary approach option chosen in HW)
BG_BUDGET_AGGR_TOTAL_COMPENS_STAFF_LEV_COMPENS_TYPE_HW = 'aggrTotalCompensAllStaffCompensHW' # 1d list of boolean; For each dynamic staff type, are we aggregating total compensation across all staffing levels and compensation types?
BG_BUDGET_COST_LEVEL_HW_ID                             = 'levelHW'                           # integer; costing section = 1, subsection = 2, or activity level = 3
BG_BUDGET_GROUP_IND_AGGR_SET_IS                        = 'aggrGroupIndIS'                    # 1d list of int; contains the master IDs of all IS indicators that the user wants to aggregate by group.
BG_BUDGET_FACILITY_IND_AGGR_SET_IS                     = 'aggrFacilityIndIS'                 # 1d list of int; contains the master IDs of all IS indicators that the user wants to aggregate by facility.
BG_BUDGET_COST_LEVEL_IS_ID                             = 'levelIS'                           # integer; costing section = 1, subsection = 2, or activity level = 3
BG_BUDGET_COST_LEVEL_HS_ID                             = 'levelHS'                           # integer; costing section = 1, subsection = 2, or activity level = 3
BG_BUDGET_COST_LEVEL_GV_ID                             = 'levelGV'                           # integer; costing section = 1, subsection = 2, or activity level = 3
BG_BUDGET_COST_LEVEL_HF_ID                             = 'levelHF'                           # integer; costing section = 1, subsection = 2, or activity level = 3
BG_BUDGET_COST_LEVEL_LG_ID                             = 'levelLG'                           # integer; costing section = 1, subsection = 2, or activity level = 3

# quick map records

BG_QUICK_MAP_BUDGET_ID        = BG_F_BUDGET_ID            # string; Numeric mstID for defaults (still cast as string). GUID for custom budgets.
BG_QUICK_MAP_MOD_ID           = BG_F_MOD_ID               # integer; Module code
BG_QUICK_MAP_TARGET_ID        = 'targetID'                # integer; budget categories vs funding sources
BG_QUICK_MAP_IND_TYPE_ID      = BG_F_IND_TYPE_ID          # integer; Range of possible values depends on module.
BG_QUICK_MAP_IS_AGGREGATE     = BG_F_IS_AGGREGATE         # boolean; Is the indicator being aggregated?
#BG_QUICK_MAP_DELIV_CHAN_ID_IC = BG_IC_F_DELIV_CHAN_ID    # string; Only applicable if BG_BUDGET_AGGR_DELIV_CHANS_IC is false
BG_QUICK_MAP_ON               = 'on'                      # boolean; True if turned on, false otherwise

# Indicator records. We add different fields according to the module represented.

BG_IND_BUDGET_ID                  = BG_F_BUDGET_ID               # string
# BG_IND_FUND_FW_ID                 = BG_F_FUND_FW_ID              # string
BG_IND_MOD_ID                     = BG_F_MOD_ID                  # integer; Module code
BG_IND_TYPE_ID                    = BG_F_IND_TYPE_ID             # integer; Range of possible values depends on module.
BG_IND_ID                         = BG_F_IND_ID                  # integer; Range of possible values depends on module.
BG_IND_BUDGET_CAT_ID              = BG_F_BUDGET_CAT_ID           # string
# BG_IND_FUND_SOURCE_ID             = BG_F_FUND_SOURCE_ID          # string
BG_IND_NAME                       = 'name'                       # string
#BG_IND_IS_AGGREGATE               = 'isAggregate'                # boolean; Is the indicator being aggregated?
#BG_IND_DISTRIB_MULTI_CATS         = 'distribMultiBudgetCats'     # boolean; Distribute indicator across multiple budget categories?
#BG_IND_DISTRIB_MULTI_FUND_SOURCES = 'distribMultiFundSources'    # boolean; Distribute indicator across multiple funding sources?
BG_IND_FUND_SOURCE_BY_FRAMEWORK   = 'fundSourceByFramework'      # dict; keys are funding funding framework IDs, values are funding source IDs (both strings)
BG_COST_CAT_IC_ID                 = 'costCatIC_ID'               # integer; ID of what we're showing in IC

BG_IND_DELIV_CHAN_ID_IC           = BG_IC_F_DELIV_CHAN_ID        # string; Only applicable if BG_BUDGET_AGGR_DELIV_CHANS_IC is false

BG_IND_COSTING_SECT_ID_PC         = BG_PC_F_COSTING_SECT_ID         # string; 
BG_IND_COSTING_SUBSECT_ID_PC      = BG_PC_F_COSTING_SUBSECT_ID      # string; 
BG_IND_ACT_ID_PC                  = BG_PC_F_ACT_ID                  # string; 
BG_IND_PROG_AREA_ID_PC            = BG_IH_F_PROG_AREA_ID            # string    

BG_IND_STAFF_LEVEL_ID_HW          = BG_F_STAFF_LEVEL_ID_HW          # int
BG_IND_COMPENS_TYPE_ID_HW         = BG_F_COMPENS_TYPE_ID_HW         # int

BG_IND_COSTING_SECT_HS_COSTING_ID         = BG_F_COSTING_SECT_HS_COSTING_ID      # string; 
BG_IND_COSTING_SUBSECT_HS_COSTING_ID      = BG_F_COSTING_SUBSECT_HS_COSTING_ID   # string; 
BG_IND_ACT_HS_COSTING_ID                  = BG_F_ACT_HS_COSTING_ID               # string; 
BG_IND_COSTING_SECT_HS_PROG_MGMT_ID       = BG_F_COSTING_SECT_HS_PROG_MGMT_ID    # string; 
BG_IND_COSTING_SUBSECT_HS_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HS_PROG_MGMT_ID # string; 
BG_IND_ACT_HS_PROG_MGMT_ID                = BG_F_ACT_HS_PROG_MGMT_ID             # string; 

BG_IND_COSTING_SECT_GV_COSTING_ID         = BG_F_COSTING_SECT_GV_COSTING_ID      # string; 
BG_IND_COSTING_SUBSECT_GV_COSTING_ID      = BG_F_COSTING_SUBSECT_GV_COSTING_ID   # string; 
BG_IND_ACT_GV_COSTING_ID                  = BG_F_ACT_GV_COSTING_ID               # string; 
BG_IND_COSTING_SECT_GV_PROG_MGMT_ID       = BG_F_COSTING_SECT_GV_PROG_MGMT_ID    # string; 
BG_IND_COSTING_SUBSECT_GV_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_GV_PROG_MGMT_ID # string; 
BG_IND_ACT_GV_PROG_MGMT_ID                = BG_F_ACT_GV_PROG_MGMT_ID             # string; 

BG_IND_COSTING_SECT_HF_COSTING_ID         = BG_F_COSTING_SECT_HF_COSTING_ID      # string; 
BG_IND_COSTING_SUBSECT_HF_COSTING_ID      = BG_F_COSTING_SUBSECT_HF_COSTING_ID   # string; 
BG_IND_ACT_HF_COSTING_ID                  = BG_F_ACT_HF_COSTING_ID               # string; 
BG_IND_COSTING_SECT_HF_PROG_MGMT_ID       = BG_F_COSTING_SECT_HF_PROG_MGMT_ID    # string; 
BG_IND_COSTING_SUBSECT_HF_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HF_PROG_MGMT_ID # string; 
BG_IND_ACT_HF_PROG_MGMT_ID                = BG_F_ACT_HF_PROG_MGMT_ID             # string; 

# budget category records

BG_BUDGET_CAT_BUDGET_ID                  = BG_F_BUDGET_ID               # string
BG_BUDGET_CAT_ID                         = BG_F_BUDGET_CAT_ID           # string; Numeric mstID for defaults (still cast as string). GUID for custom budgets.
BG_BUDGET_CAT_NAME                       = 'name'                       # string
BG_BUDGET_CAT_STR_CONST                  = 'stringConstant'             # string
BG_BUDGET_CAT_TOTAL_COST                 = 'totalCost'                  # list of float; by year
BG_BUDGET_CAT_CUSTOM                     = 'custom'                     # boolean

# budget category share rec

BG_BUDGET_CAT_SHARE_BUDGET_ID     = BG_F_BUDGET_ID         # string
BG_BUDGET_CAT_SHARE_MOD_ID        = BG_F_MOD_ID            # integer; Module code
BG_BUDGET_CAT_SHARE_IND_TYPE_ID   = BG_F_IND_TYPE_ID       # integer; Range of possible values depends on module.
BG_BUDGET_CAT_SHARE_IND_ID        = BG_F_IND_ID
BG_BUDGET_CAT_SHARE_BUDGET_CAT_ID = BG_F_BUDGET_CAT_ID     # string
BG_BUDGET_CAT_SHARE_VALUE         = 'value'                # float

BG_BUDGET_CAT_SHARE_DELIV_CHAN_ID = BG_IC_F_DELIV_CHAN_ID  # string

BG_BUDGET_CAT_SHARE_COSTING_SECT_ID_PC         = BG_PC_F_COSTING_SECT_ID         # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_ID_PC      = BG_PC_F_COSTING_SUBSECT_ID      # string; 
BG_BUDGET_CAT_SHARE_ACT_ID_PC                  = BG_PC_F_ACT_ID                  # string; 
BG_BUDGET_CAT_SHARE_PROG_AREA_ID_PC            = BG_IH_F_PROG_AREA_ID            # string    

BG_BUDGET_CAT_SHARE_STAFF_LEVEL_ID_HW  = BG_F_STAFF_LEVEL_ID_HW    # int
BG_BUDGET_CAT_SHARE_COMPENS_TYPE_ID_HW = BG_F_COMPENS_TYPE_ID_HW   # int

BG_BUDGET_CAT_SHARE_COSTING_SECT_HS_COSTING_ID         = BG_F_COSTING_SECT_HS_COSTING_ID      # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HS_COSTING_ID      = BG_F_COSTING_SUBSECT_HS_COSTING_ID   # string; 
BG_BUDGET_CAT_SHARE_ACT_HS_COSTING_ID                  = BG_F_ACT_HS_COSTING_ID               # string; 
BG_BUDGET_CAT_SHARE_COSTING_SECT_HS_PROG_MGMT_ID       = BG_F_COSTING_SECT_HS_PROG_MGMT_ID    # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HS_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HS_PROG_MGMT_ID # string; 
BG_BUDGET_CAT_SHARE_ACT_HS_PROG_MGMT_ID                = BG_F_ACT_HS_PROG_MGMT_ID             # string; 

BG_BUDGET_CAT_SHARE_COSTING_SECT_GV_COSTING_ID         = BG_F_COSTING_SECT_GV_COSTING_ID      # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_GV_COSTING_ID      = BG_F_COSTING_SUBSECT_GV_COSTING_ID   # string; 
BG_BUDGET_CAT_SHARE_ACT_GV_COSTING_ID                  = BG_F_ACT_GV_COSTING_ID               # string; 
BG_BUDGET_CAT_SHARE_COSTING_SECT_GV_PROG_MGMT_ID       = BG_F_COSTING_SECT_GV_PROG_MGMT_ID    # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_GV_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_GV_PROG_MGMT_ID # string; 
BG_BUDGET_CAT_SHARE_ACT_GV_PROG_MGMT_ID                = BG_F_ACT_GV_PROG_MGMT_ID             # string; 

BG_BUDGET_CAT_SHARE_COSTING_SECT_HF_COSTING_ID         = BG_F_COSTING_SECT_HF_COSTING_ID      # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HF_COSTING_ID      = BG_F_COSTING_SUBSECT_HF_COSTING_ID   # string; 
BG_BUDGET_CAT_SHARE_ACT_HF_COSTING_ID                  = BG_F_ACT_HF_COSTING_ID               # string; 
BG_BUDGET_CAT_SHARE_COSTING_SECT_HF_PROG_MGMT_ID       = BG_F_COSTING_SECT_HF_PROG_MGMT_ID    # string; 
BG_BUDGET_CAT_SHARE_COSTING_SUBSECT_HF_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HF_PROG_MGMT_ID # string; 
BG_BUDGET_CAT_SHARE_ACT_HF_PROG_MGMT_ID                = BG_F_ACT_HF_PROG_MGMT_ID             # string; 

# funding framework records

BG_FUND_FW_ID                  = BG_F_FUND_FW_ID    # string; Numeric mstID for defaults (still cast as string). GUID for custom budgets.
BG_FUND_FW_NAME                = 'name'             # string
BG_FUND_FW_STR_CONSTANT        = 'stringConstant'   # string
BG_FUND_FW_CUSTOM              = 'custom'           # boolean

# funding source records

BG_FUND_SOURCE_FW_ID        = BG_F_FUND_FW_ID         # string
BG_FUND_SOURCE_ID            = BG_F_FUND_SOURCE_ID     # string; Numeric mstID for defaults (still cast as string). GUID for custom budgets.
BG_FUND_SOURCE_NAME          = 'name'                  # string
BG_FUND_SOURCE_STR_CONSTANT  = 'stringConstant'        # string
BG_FUND_SOURCE_CUSTOM        = 'custom'                # boolean

# funding source share rec
BG_FUND_SOURCE_SHARE_BUDGET_ID       = BG_F_BUDGET_ID         # string
BG_FUND_SOURCE_SHARE_MOD_ID          = BG_F_MOD_ID            # integer; Module code
BG_FUND_SOURCE_SHARE_IND_TYPE_ID     = BG_F_IND_TYPE_ID       # integer; Range of possible values depends on module.
BG_FUND_SOURCE_SHARE_IND_ID          = BG_F_IND_ID            # string
BG_FUND_SOURCE_SHARE_FW_ID           = BG_F_FUND_FW_ID        # string
BG_FUND_SOURCE_SHARE_FUND_SOURCE_ID  = BG_F_FUND_SOURCE_ID    # string
BG_FUND_SOURCE_SHARE_VALUE           = 'value'                # float         #??? 'values'             # list of float by year

BG_FUND_SOURCE_SHARE_DELIV_CHAN_ID   = BG_IC_F_DELIV_CHAN_ID  # string

BG_FUND_SOURCE_SHARE_COSTING_SECT_ID_PC         = BG_PC_F_COSTING_SECT_ID         # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_ID_PC      = BG_PC_F_COSTING_SUBSECT_ID      # string; 
BG_FUND_SOURCE_SHARE_ACT_ID_PC                  = BG_PC_F_ACT_ID                  # string; 
BG_FUND_SOURCE_SHARE_PROG_AREA_ID_PC            = BG_IH_F_PROG_AREA_ID            # string    

BG_FUND_SOURCE_SHARE_STAFF_LEVEL_ID_HW   = BG_F_STAFF_LEVEL_ID_HW    # int
BG_FUND_SOURCE_SHARE_COMPENS_TYPE_ID_HW  = BG_F_COMPENS_TYPE_ID_HW   # int

BG_FUND_SOURCE_SHARE_COSTING_SECT_HS_COSTING_ID         = BG_F_COSTING_SECT_HS_COSTING_ID      # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HS_COSTING_ID      = BG_F_COSTING_SUBSECT_HS_COSTING_ID   # string; 
BG_FUND_SOURCE_SHARE_ACT_HS_COSTING_ID                  = BG_F_ACT_HS_COSTING_ID               # string; 
BG_FUND_SOURCE_SHARE_COSTING_SECT_HS_PROG_MGMT_ID       = BG_F_COSTING_SECT_HS_PROG_MGMT_ID    # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HS_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HS_PROG_MGMT_ID # string; 
BG_FUND_SOURCE_SHARE_ACT_HS_PROG_MGMT_ID                = BG_F_ACT_HS_PROG_MGMT_ID             # string; 

BG_FUND_SOURCE_SHARE_COSTING_SECT_GV_COSTING_ID         = BG_F_COSTING_SECT_GV_COSTING_ID      # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_GV_COSTING_ID      = BG_F_COSTING_SUBSECT_GV_COSTING_ID   # string; 
BG_FUND_SOURCE_SHARE_ACT_GV_COSTING_ID                  = BG_F_ACT_GV_COSTING_ID               # string; 
BG_FUND_SOURCE_SHARE_COSTING_SECT_GV_PROG_MGMT_ID       = BG_F_COSTING_SECT_GV_PROG_MGMT_ID    # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_GV_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_GV_PROG_MGMT_ID # string; 
BG_FUND_SOURCE_SHARE_ACT_GV_PROG_MGMT_ID                = BG_F_ACT_GV_PROG_MGMT_ID             # string; 

BG_FUND_SOURCE_SHARE_COSTING_SECT_HF_COSTING_ID         = BG_F_COSTING_SECT_HF_COSTING_ID      # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HF_COSTING_ID      = BG_F_COSTING_SUBSECT_HF_COSTING_ID   # string; 
BG_FUND_SOURCE_SHARE_ACT_HF_COSTING_ID                  = BG_F_ACT_HF_COSTING_ID               # string; 
BG_FUND_SOURCE_SHARE_COSTING_SECT_HF_PROG_MGMT_ID       = BG_F_COSTING_SECT_HF_PROG_MGMT_ID    # string; 
BG_FUND_SOURCE_SHARE_COSTING_SUBSECT_HF_PROG_MGMT_ID    = BG_F_COSTING_SUBSECT_HF_PROG_MGMT_ID # string; 
BG_FUND_SOURCE_SHARE_ACT_HF_PROG_MGMT_ID                = BG_F_ACT_HF_PROG_MGMT_ID             # string; 


# Fields used in RP data structures

RP_SECTOR_ID = 'sectorID'
# RP_SECTOR_ON = 'on'
RP_SECTOR_GOAL_RECORDS = 'goalRecords'

RP_SECTOR_EDU_PRIM_SCH_START_AGE                  = 'primarySchoolStartAge'            # int
RP_SECTOR_EDU_PRIM_SCH_END_AGE                    = 'primarySchoolEndAge'              # int
RP_SECTOR_EDU_SEC_SCH_START_AGE                   = 'secondarySchoolStartAge'          # int
RP_SECTOR_EDU_SEC_SCH_END_AGE                     = 'secondarySchoolEndAge'            # int
# RP_SECTOR_EDU_STUDENT_PER_TEACHER_PRIM           = 'studentsPerTeacherPrimary'        # float
RP_SECTOR_EDU_AVG_ANN_TEACHER_SAL_PRIM_GDP_RATIO  = 'avgAnnualTeacherSalaryPrimaryGDPRatio'    # int
# RP_SECTOR_EDU_AVG_ANN_TEACHER_SAL_SEC            = 'avgAnnualTeacherSalarySecondary'  # int
RP_SECTOR_EDU_PRIM_SPEND_PER_STUDENT_PERC_GDP     = 'primarySpendPerStudentPercGDP'     # int
RP_SECTOR_EDU_SEC_SPEND_PER_STUDENT_PERC_GDP      = 'secondarySpendPerStudentPercGDP'   # int

RP_SECTOR_HLTH_AVG_UNIT_COST_SKILLED_DELIV     = 'avgUnitCostSkilledDelivery'    # float
RP_SECTOR_HLTH_AVG_UNIT_COST_ANC_4_PLUS_VISITS = 'avgUnitCostANC4PlusVisits'     # float
RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_DOCTORS  = 'percSkilledHealthProfDoctors'  # int
RP_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_NURSES   = 'percSkilledHealthProfNurses'   # int
RP_SECTOR_HLTH_AVG_SALARY_DOCTORS              = 'avgSalaryDoctors'              # float
RP_SECTOR_HLTH_AVG_SALARY_NURSES               = 'avgSalaryNurses'               # float

RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_WATER     = 'capCostsPerHHImprovedWater'       # float
RP_SECTOR_WASH_ANN_COST_MENSTRUAL_HLTH_COMM    = 'annualCostMenstrualHealthComm'    # float
RP_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_SANI      = 'capCostsPerHHImprovedSanitation'  # float

RP_SECTOR_FOOD_PRIM_STAPLE_CROP                    = 'primaryStapleCrop'                    # string
RP_SECTOR_FOOD_AGRICULT_LAND                       = 'agriculturalLand'                     # int
RP_SECTOR_FOOD_CROP_LAND                           = 'cropLand'                             # int
RP_SECTOR_FOOD_STAPLE_CROP_PROD_BASELINE           = 'stapleCropProductionBaseline'         # int
RP_SECTOR_FOOD_AREA_HARV_STAPLE_CROP               = 'areaHarvestedStapleCrop'              # int
RP_SECTOR_FOOD_TONNES_HARV_PERC_HECT               = 'tonnesHarvestedPerHectare'            # float
RP_SECTOR_FOOD_ANN_KG_PER_YEAR_CONSUMP_STAPLE_CROP = 'annualKgPerYearConsumptionStapleCrop' # int
RP_SECTOR_FOOD_PROD_COSTS_STAPLE_CROP              = 'productionCostsStapleCrop'            # float
RP_SECTOR_FOOD_DAILY_COST_FOOD_AID                 = 'dailyCostFoodAid'                     # float

RP_SECTOR_URB_PERC_POP_URB_BASELINE_YEAR    = 'percPopUrbanBaselineYear' # int
RP_SECTOR_URB_PERC_POP_URB_ENDLINE_YEAR     = 'percPopUrbanEndlineYear'  # int
RP_SECTOR_URB_COST_PER_HH_MOVE_TO_HOUSING   = 'costPerHHMoveToHousing'   # float

RP_SECTOR_ENERGY_KWH_PROD_PER_CAP_PER_YEAR        = 'KWhProducedPerCapitaPerYear'  # float
RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CONNECT_GRID    = 'capCostsPerHHConnectGrid'     # int
RP_SECTOR_ENERGY_CAP_COSTS_PER_HH_CLEAN_COOKING   = 'capCostPerHHCleanCooking'     # float
RP_SECTOR_ENERGY_COST_PER_KWH                     = 'costPerKWh'                   # float

RP_SECTOR_ECO_GDP_PER_CAPITA    = 'GDPPerCapita'                # float
# RP_SECTOR_ECO_BASELINE_GDP      = 'baselineGDP'               # int
# RP_SECTOR_ECO_GDP_GROWTH_RATE   = 'GDPGrowthRate'             # float
RP_SECTOR_ECO_LABOR_PARTIC_RATE = 'laborParticipationRate'      # float
RP_SECTOR_ECO_CHANGE_GDP_CDR    = 'changeInGDPPerPercChangeCDR' # float

# RP_SECTOR_BYO_POP                = 'population'      # string
RP_SECTOR_BYO_INTERV             = 'intervention'    # string
RP_SECTOR_BYO_TARG_POP           = 'targPop'         # int; 
# RP_SECTOR_BYO_UNIT_OF_ANALYSIS_1 = 'unitOfAnalysis1' # string
RP_SECTOR_BYO_MULTIPLIER_1       = 'multiplier1'     # int
RP_SECTOR_BYO_UNIT_OF_ANALYSIS_2 = 'unitOfAnalysis2' # string
RP_SECTOR_BYO_UNIT_COST_2        = 'unitCost2'       # float

RP_CLIM_BASELINE_MEAN_MED_TEMP              = 'baselineMeanMedianTemp'                       # float
RP_CLIM_PERC_CHANGE_GDP_1_DEG_WARM          = 'percChangeGDP1DegreeWarming'                  # float; can be negative
RP_CLIM_PERC_CHANGE_ENERGY_NEEDS_1_DEG_WARM = 'percChangeEnergyNeeds1DegreeWarming'          # float; can be negative
RP_CLIM_PERC_CHANGE_STAPLE_CROP_1_DEG_WARM  = 'percChangeStapleCroplandNeeds1DegreeWarming'  # float; can be negative
RP_CLIM_PERC_CHANGE_STAPLE_CROP_2_DEG_WARM  = 'percChangeStapleCroplandNeeds2DegreesWarming' # float; can be negative
RP_CLIM_PERC_CHANGE_STAPLE_CROP_3_DEG_WARM  = 'percChangeStapleCroplandNeeds3DegreesWarming' # float; can be negative

RP_GOAL_ON                  = 'on'                # bool
RP_GOAL_ID                  = 'goalID'            # int
RP_GOAL_SELECTED_SUBGOAL_ID = 'selectedSubgoalID' # int; for MV
RP_GOAL_DEFAULT_SUBGOAL_ID  = 'defaultSubgoalID'  # int; for meta MV
RP_GOAL_SUBGOAL_RECORDS     = 'subgoalRecords'    # list of dict
# RP_GOAL_INTERP_TYPE       = 'interpTypeID'    # int
RP_GOAL_RESULTING_CHANGE    = 'resultingChange' # list of int; only used for Build Your Own Sector, which is the only sector without subgoals (where resultingChange is normally stored).

RP_SUBGOAL_ID               = 'subgoalID'
RP_SUBGOAL_RESULTING_CHANGE = 'resultingChange' # list of int

RP_FP_SCEN_SC_ACCELERATED = 'accelerated'
RP_FP_SCEN_SC_MEDIAN      = 'median'
RP_FP_SCEN_SC_FLATLINE    = 'flatline'

# Used for various result MVs to separate portions of MVs into status quo, goal, and counterfactual. Sometimes results will have statusQuo, 
# sometimes they won't
RP_RES_FP_SCEN_STATUS_QUO     = 'statusQuo'
RP_RES_FP_SCEN_GOAL           = 'goal'
RP_RES_FP_SCEN_COUNTERFACTUAL = 'counterfactual'

RP_RES_SUM_CUMU_COSTS                      = 'cumulativeCosts'
RP_RES_SUM_DIRECT                          = 'direct'
RP_RES_SUM_INDIRECT                        = 'indirect'
RP_RES_SUM_CUMU_SAVINGS                    = 'cumulativeSavings'
RP_RES_SUM_EDU_SECT_PRIM                   = 'eduSectorPrimary'
RP_RES_SUM_EDU_SECT_SEC                    = 'eduSectorSecondary'
RP_RES_SUM_HLTH_SECT_SKILLED_DELIV         = 'healthSectorSkilledDeliveries'
RP_RES_SUM_HLTH_SECT_ANC                   = 'healthSectorANC'
RP_RES_SUM_WASH_SECT_IMPR_WATER            = 'WASHSectorImprovedWater'
RP_RES_SUM_WASH_SECT_IMPR_SANI             = 'WASHSectorImprovedSanitation'
RP_RES_SUM_FOOD_SEC_SECT_RED_FOOD_INSEC    = 'foodSecSectorReduceFoodInsecurity'
RP_RES_SUM_URB_SECT_REDUCE_INFORM_SETT     = 'urbSectorReduceInformalSettlements'
RP_RES_SUM_ENERGY_SECT_INCR_ACC_ELEC       = 'energySectorIncreaseAccessElectricity'
RP_RES_SUM_ENERGY_SECT_INCR_ACC_CLEAN_COOK = 'energySectorIncreaseAccessCleanCooking'
RP_RES_SUM_SAVINGS_PER_DOLLAR_INVEST       = 'savingsPerDollarInvested'

# Sources (for Sector goals and Review defaults; since they are not guaranteed to be a 1-1 correspondence between the fields there and the available
# sources, we store them in a separate MV)

RP_SRC_HH_SIZE                               = 'householdSizeSource'                    # str
RP_SRC_SECTOR_EDU_SCHOOL_AGE_RANGES          = 'schoolAgeRangesSource'                  # str
RP_SRC_FP_METHOD_MIX                         = 'FPMethodMixSource'                      # str; only for revised sources; default comes from FamPlan
RP_SRC_FP_EFFECTIVENESS                      = 'FPEffectivenessSource'                  # str; only for revised sources; default comes from FamPlan
RP_SRC_FP_UNIT_COSTS                         = 'FPUnitCostsSource'                      # str
RP_SRC_DP_FIRST_YEAR_POP                     = 'DPFirstYearPopSource'                   # str
RP_SRC_FP_TFR                                = 'FP_TFRSource'                           # str

RP_SRC_SECTOR_EDU_AVG_ANN_TEACHER_SAL_PRIM   = 'avgAnnualTeacherSalaryPrimarySource'    # str
RP_SRC_SECTOR_EDU_ANN_COST_EDU_PRIM_STUDENT  = 'annualCostEducPrimaryStudentSource'     # str
RP_SRC_SECTOR_EDU_ANN_COST_EDU_SEC_STUDENT   = 'annualCostEducSecondaryStudentSource'   # str

RP_SRC_SECTOR_HLTH_AVG_UNIT_COST_SKILLED_DELIV     = 'avgUnitCostSkilledDeliverySource'    # str
RP_SRC_SECTOR_HLTH_AVG_UNIT_COST_ANC_4_PLUS_VISITS = 'avgUnitCostANC4PlusVisitsSource'     # str
RP_SRC_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_DOCTORS  = 'percSkilledHealthProfDoctorsSource'  # str
RP_SRC_SECTOR_HLTH_PERC_SKILLED_HLTH_PROF_NURSES   = 'percSkilledHealthProfNursesSource'   # str
RP_SRC_SECTOR_HLTH_AVG_SALARY_DOCTORS              = 'avgSalaryDoctorsSource'              # str
RP_SRC_SECTOR_HLTH_AVG_SALARY_NURSES               = 'avgSalaryNursesSource'               # str

RP_SRC_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_WATER     = 'capCostsPerHHImprovedWaterSource'       # str
RP_SRC_SECTOR_WASH_CAP_COSTS_PER_HH_IMPR_SANI      = 'capCostsPerHHImprovedSanitationSource'  # str

RP_SRC_SECTOR_FOOD_PRIM_STAPLE_CROP                    = 'primaryStapleCropSource'                      # str
RP_SRC_SECTOR_FOOD_CROP_LAND                           = 'cropLandSource'                               # str
RP_SRC_SECTOR_FOOD_STAPLE_CROP_PROD_BASE               = 'stapleCropProductionBaselineSource'           # str
RP_SRC_SECTOR_FOOD_TONNES_HARV_PERC_HECT               = 'tonnesHarvestedPerHectareSource'              # str
RP_SRC_SECTOR_FOOD_ANN_KG_PER_YEAR_CONSUMP_STAPLE_CROP = 'annualKgPerYearConsumptionStapleCropSource'   # str
RP_SRC_SECTOR_FOOD_PROD_COSTS_STAPLE_CROP              = 'productionCostsStapleCropSource'              # str
RP_SRC_SECTOR_FOOD_DAILY_COST_FOOD_AID                 = 'dailyCostFoodAidSource'                       # str

RP_SRC_SECTOR_URB_PERC_POP_URB_BASE_END         = 'percPopUrbanBaselineEndlineSource'  # str
RP_SRC_SECTOR_URB_COST_PER_HH_MOVE_TO_HOUSING   = 'costPerHHMoveToHousingSource'       # str

RP_SRC_SECTOR_ENERGY_CAP_COSTS_PER_HH_CONNECT_GRID    = 'capCostsPerHHConnectGridSource'     # str
RP_SRC_SECTOR_ENERGY_CAP_COSTS_PER_HH_CLEAN_COOKING   = 'capCostPerHHCleanCookingSource'     # str

RP_SRC_SECTOR_ECO_GDP_PER_CAPITA    = 'GDPPerCapitaSource'                # str
RP_SRC_SECTOR_ECO_LABOR_PARTIC_RATE = 'laborParticipationRateSource'      # str
RP_SRC_SECTOR_ECO_CHANGE_GDP_CDR    = 'changeInGDPPerPercChangeCDRSource' # str

RP_SRC_GOALS_PRIMARY_ENROLL_ALL          = 'primaryEnrollmentAllSource'
RP_SRC_GOALS_PRIMARY_ENROLL_MALE         = 'primaryEnrollmentMaleSource'
RP_SRC_GOALS_PRIMARY_ENROLL_FEMALE       = 'primaryEnrollmentFemaleSource'
RP_SRC_GOALS_SECONDARY_ENROLL_ALL        = 'secondaryEnrollmentAllSource'
RP_SRC_GOALS_SECONDARY_ENROLL_MALE       = 'secondaryEnrollmentMaleSource'
RP_SRC_GOALS_SECONDARY_ENROLL_FEMALE     = 'secondaryEnrollmentFemaleSource'
RP_SRC_GOALS_STUDENT_TEACHER_RATIO       = 'studentTeacherRatioSource'
RP_SRC_GOALS_BIRTHS_ATTEND_SKILLED_PERS  = 'birthsAttendedBySkilledPersonnelSource'
RP_SRC_GOALS_PREG_W_4_PLUS_ANC_VISITS    = 'pregnantWomenRec4PlusANCVisitsSource'
RP_SRC_GOALS_HLTH_PROF_1000_POP          = 'healthProfPer1000PopSource'
RP_SRC_GOALS_PERC_ACCESS_IMP_WATER       = 'percAccessImprovedWaterSource'
RP_SRC_GOALS_PERC_ACCESS_IMP_SANI        = 'percAccessImprovedSanitationSource'
RP_SRC_GOALS_PERC_PROD_STAPLE_CROP       = 'percDomesticProductionStapleCropSource'
RP_SRC_GOALS_PERC_SEV_FOOD_INSEC         = 'percSevereFoodInsecuritySource'
RP_SRC_GOALS_PERC_URB_POP_INFORM_SET     = 'percUrbanPopInformalSettlementsSource'
RP_SRC_GOALS_PERC_POP_ACCESS_ELEC        = 'percPopAccessElectricitySource'
RP_SRC_GOALS_KWH_PRODUCED                = 'KWhProducedPerPersonPerYearSource'
RP_SRC_GOALS_PERC_ACCESS_CLEAN_COOK_TECH = 'percAccessCleanCookingTechSource'
RP_SRC_GOALS_UNEMPLOYMENT_RATE           = 'unemploymentRateSource'
RP_SRC_GOALS_PERC_NEET_ALL               = 'percNEETAllSource'
RP_SRC_GOALS_PERC_NEET_MALES             = 'percNEETMalesSource'
RP_SRC_GOALS_PERC_NEET_FEMALES           = 'percNEETFemalesSource'

RP_SRC_CLIM_BASELINE_MEAN_MED_TEMP              = 'baselineMeanMedianTempSource'                       # str
RP_SRC_CLIM_PERC_CHANGE_GDP_1_DEG_WARM          = 'percChangeGDP1DegreeWarmingSource'                  # str
RP_SRC_CLIM_PERC_CHANGE_ENERGY_NEEDS_1_DEG_WARM = 'percChangeEnergyNeeds1DegreeWarmingSource'          # str
RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_1_DEG_WARM  = 'percChangeStapleCroplandNeeds1DegreeWarmingSource'  # str
RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_2_DEG_WARM  = 'percChangeStapleCroplandNeeds2DegreesWarmingSource' # str
RP_SRC_CLIM_PERC_CHANGE_STAPLE_CROP_3_DEG_WARM  = 'percChangeStapleCroplandNeeds3DegreesWarmingSource' # str

RP_SSP_BASE = 'base'
RP_SSP1     = 'SSP1'
RP_SSP2     = 'SSP2'
RP_SSP3     = 'SSP3'
RP_SSP5     = 'SSP5'





from pydantic import BaseModel
from typing import Any, List

'''
	TransactionRecord 

	See TRConst in Spectrum Transaction Server for a list of possible values that targets and commands can be.
	
	target: 
		String representing type of item to be modified.

	command:
		String representing type of transaction to perform.
		
	ID: 
		Master ID / key to identify item to be changed as a string.
	
	name: 
		Item name.

	SpeciaL:
		progAreaID: 
			For ProgramAreaSubgroups and Interventions targets only. Ignore otherwise.

		subgroupID:
			For Interventions only. Ignore otherwise.
                                                                
		value:
			Value to be set by transaction. The only value we do not set using this field is name.     

'''
# Base class for any transaction record
class TransactionRecord(BaseModel): 
	target     : str = ''
	command    : str = ''
	params     : Any = {} # Prefer this to value when possible. Object with all needed fields. 

	ID         : str = '' 
	name       : str = ''

	# Special
	progAreaID : str = '' # *See below
	subgroupID : str = '' # *See below
	value      : Any = ''

# *Can we make this a part of value at some point? WHY: There is more than just progAreaID and subgroupID and I'd rather not flood this
# base class with fields every time we need new ones

'''
	ModVarRecord

	module:
		Two-letter module code. Ex: DP

	modvars:
		List of ModVar tags. Ex: ['<DP_Tag1_V1>', '<DP_Tag2_V1>']
'''

class ModVarRecord(BaseModel):
	module  : str = ''
	modvars : List[str] = ['']

# Base class for any list of transactions
class Transactions(BaseModel):
	projID           : str = ''
	clientModvarList : List[ModVarRecord] = [dict(ModVarRecord())]
	transactions     : List[TransactionRecord] = [dict(TransactionRecord())]




from scipy.io import savemat
import numpy as np
import pandas as pd
#import os
import scipy.io
import openpyxl

import .InterventionInit
import matplotlib.pyplot as plt

from .alloc_parameters import alloc_parameters
from .make_model3 import make_model3
import pickle
import scipy
import numpy as np
import json
import math
from .get_distribution_fns import get_distribution_fns
from scipy import interpolate

#from InterventionScenario import InterventionScenario
from .InterventionInit import InterventionInit

from .get_objective import get_objective as f_obj
from .ode15s import ode15s, create_steps
from .goveqs_scaleup import goveqs_scaleup

import time

def run_simulate_forward(ISO3= 'NGA', data_glob_input=None):
   
    Model_setup_file_name = "data/SetupModel/Model_setup.pkl"
    with open(Model_setup_file_name, 'rb') as fp:
        setup_data = pickle.load(fp)

    xi = setup_data['xi'].copy()
    i0 = setup_data['i0']
    s0 = setup_data['s0']
    gps0 = setup_data['gps0']
    sel0 = setup_data['sel0']
    agg0 = setup_data['agg0'] 
    d0 = setup_data['d0']
    ref = setup_data['ref']  
    agg = setup_data['agg']
    gps = setup_data['gps']
    gps1 = setup_data['gps1']
    gps2 = setup_data['gps2']
    sel = setup_data['sel']

    ScnPARS = [ISO3]
    jso_file_name = "data/CtriesDefaults/" + ISO3 + "_calibres.json"

    if data_glob_input==None:
        f_glob_input = open('TBExample.JSON')
        data_glob_input = json.load(f_glob_input)

    expl_prm = data_glob_input['<TB_DynamicalModelPRM_V1>']
    expl_p = data_glob_input['<TB_DynamicalModelPRM_V1>']['p']
    expl_r = data_glob_input['<TB_DynamicalModelPRM_V1>']['r']
    
    cs_global = expl_r['cs']
    cs2_global = expl_r['cs2']
    pu_global = expl_p['pu']

    #Loading ctr specific data
    f_data_ctr = open(jso_file_name)
    data_ctr = json.load(f_data_ctr)
    data = data_ctr['data']
    prm = data_ctr['prm']
    r = data_ctr['r']
    p = data_ctr['p']
    xs = np.array(data_ctr['xs'])
    prm['r']['self_cure'] = np.array([0.1667,        0,   0.1667])
    
    prm['r']['cs'] =  cs_global
    prm['r']['cs2'] =  cs2_global
    prm['p']['pu'] = pu_global
    r = prm['r']

    show = 0
    f0 = get_distribution_fns(data['sym'],   'lognorm', show) #f0 = get_distribution_fns(data.sym,         'lognorm', show);#f0 = get_distribution_fns(data.inc2015,   'lognorm', show)
    
    #I add this here so that the code can run, Carel should update 'data' in the JSON file     
    popn          = data['pop00_22'][-1] #popn          = data.pop00_22(end); % 2022
    data['inc2015']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-8,:] #data.inc2015  = data.inc_all(end-7,:); 
    data['inc2022']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-1,:] #data.inc2022  = data.inc_all(end,:); 
    data['inc2000'] = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-23,:]
    #% data.inc_h1  = data.inc_h1;
    #% data.mdr2015  = data.mdr2015; 
    #% data.mdr2022  = data.mdr2022;
    #data['mort_H0']  = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-1,:] #data.mort_H0  = data.mort_H0(end,:);
    data['mort_H02000'] = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-23,:]
    data['mort_H02022'] = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-1,:]

    data['mort_H12022'] = np.array(data['mort_H1'])
    data['noti']     = np.array([0.8, 1, 1.2])*np.array(data['noti']) #data.noti     = [0.8 1 1.2]*data.noti;
    data['mdriniTX'] = np.array([0.8, 1, 1.2])*np.array(data['mdriniTX']) #data.mdriniTX = [0.8 1 1.2]*data.mdriniTX; 
    data['sym']      = np.array([0.36, 0.50, 0.80])#data.sym      = [0.36 0.50 0.80];        % proportion of prevalent TB that has symptoms

    #% Get the HIV data                                                         
    prm['ART_start'] = data['ART_start'] #prm.ART_start = data.ART_start;                     %ART_start;
    data['ART_covg'] =  np.array([0.85, 1, 1.15])*data['ART_covg'] #data.ART_covg =  [0.85 1 1.15]*data.ART_covg;       %ARTcovg_2022;
    data['HIV_prev'] = np.array([0.85, 1, 1.15])*(data['HIV_prev']/popn) #data.HIV_prev = [0.85 1 1.15]*(data.HIV_prev./popn);%HIVprev_2022 (proportion of population)
    HIV_incd = data['HIV_incd'] #HIV_incd = data.HIV_incd;                           %HIV incidence (proportion of population) 1980-2022
    
    data['noti2022'] = data['noti']*np.array([0.8, 1, 1.2])
    data['noti_cu'] = sum(data['c_newinc'])*np.array([0.8, 1, 1.2])

    f1 = get_distribution_fns(data['inc2000'],     'lognorm', show) #f1 = get_distribution_fns(data.inc2000,     'lognorm', show);  
    f2 = get_distribution_fns(data['inc2022'],     'lognorm', show) #f2 = get_distribution_fns(data.inc2022,     'lognorm', show);
    f3 = get_distribution_fns(data['mort_H02000'], 'lognorm', show) #f3 = get_distribution_fns(data.mort_H02000, 'lognorm', show);
    f4 = get_distribution_fns(data['mort_H02022'], 'lognorm', show)#f4 = get_distribution_fns(data.mort_H02022, 'lognorm', show);
    f5 = get_distribution_fns(data['noti_cu'],     'lognorm', show)#f5 = get_distribution_fns(data.noti_cu,     'lognorm', show);
    f6 = get_distribution_fns(data['noti2022'],    'lognorm', show)#f6 = get_distribution_fns(data.noti2022,    'lognorm', show);
    f7 = get_distribution_fns(data['mdr2015'],     'lognorm', show)#f7 = get_distribution_fns(data.mdr2015,     'lognorm', show);
    f8 = get_distribution_fns(data['mdr2022'],     'lognorm', show)#f8 = get_distribution_fns(data.mdr2022,     'lognorm', show);
    f9 = get_distribution_fns(data['mdriniTX'],    'lognorm', show)#f9 = get_distribution_fns(data.mdriniTX,    'lognorm', show); 
    f10 = get_distribution_fns(data['HIV_prev'],   'lognorm', show)#f10 = get_distribution_fns(data.HIV_prev,   'lognorm', show);
    f11 = get_distribution_fns(data['inc_h1'],     'lognorm', show) #f11 = get_distribution_fns(data.inc_h1,     'lognorm', show);
    f12 = get_distribution_fns(data['ART_covg'],   'lognorm', show)#f12 = get_distribution_fns(data.ART_covg,   'lognorm', show);
    f13 = get_distribution_fns(data['mort_H12022'],'lognorm', show)#f13 = get_distribution_fns(data.mort_H12022,'lognorm', show);

    #lhd.fn  = @(sym,inc2000, inc2022, mort_H02000,mort_H02022,noti_cu, noti2022, mdr2015, mdr2022, mdriniTX,HIV_prev,inc_h1,ART_covg, mort_H12022)f0(sym)+f1(inc2000)+ 10*f2(inc2022)+f3(mort_H02000)+ 10*f4(mort_H02022)+ f5(noti_cu)+f6(noti2022)+f7(mdr2015)+f8(mdr2022)+f9(mdriniTX)+f10(HIV_prev)+f11(inc_h1)+f12(ART_covg)+f13(mort_H12022);
    def fn(sym,inc2000, inc2022, mort_H02000,mort_H02022,noti_cu, noti2022, mdr2015, mdr2022, mdriniTX,HIV_prev,inc_h1,ART_covg, mort_H12022):   
            res_0 = f0[0](sym)
            res_1 = f1[0](inc2000) 
            res_2 = 10*f2[0](inc2022)
            res_3 = f3[0](mort_H02000) 
            res_4 = 10*f4[0](mort_H02022) 
            res_5 = f5[0](noti_cu)
            res_6 = f6[0](noti2022)
            res_7 = f7[0](mdr2015)
            res_8 = f8[0](mdr2022)
            res_9 = f9[0](mdriniTX)
            res_10 = f10[0](HIV_prev)
            res_11 = f11[0](inc_h1)
            res_12 = f12[0](ART_covg)
            res_13 = f13[0](mort_H12022)
            result = res_0 + res_1 + res_2 + res_3 + res_4 + res_5 + res_6 + res_7 + res_8 + res_9 + res_10 + res_11 + res_12 + res_13
            return result

    lhd = {}
    lhd['fn']  = fn  
    lhd['sgn'] = -math.inf
 
    def obj(x):
        [r_out, r_aux] = f_obj(x, prm.copy(), ref, sel, agg, gps, gps1, gps2,lhd)
        return([r_out,r_aux]) 


    #clear all; %load calibres2; xsto = xsto4;    
    #load calibres1; xsto = xsto2; %(2e4:end,:);
    #[M,I] = max(outsto2); %(2e4:end,:));
    # load Model_setup;
    #xs = xsto(I,:);  % Most probable parameter set 
    #ix0 = 2e4; nx = 150; dx = round((size(xsto,1)-ix0)/nx); %2e4 nx = 150
    #xs_all = xsto(ix0:dx:end,:,1);
    #pct_xs= prctile(xs_all,[2.5,50,97.5])


    #% % --- Get the Regional notifications
    tend1   = 2024+1 #;  % Intervention starts except vaccination  
    tend1b  = 2028#;  % Vaccination starts
    tend2   = 2035+1#;  % End date for simulation

    #% --- Do the simulations 
    #opts = odeset('NonNegative',[1:i0.nstates],'Refine',64,'AbsTol',1e-10,'RelTol',1e-10);

    r0 = r.copy(); p0 = p.copy()
    inct = []
    noti = []
    inch1 = []
    mdr = []
    morth0 = []
    morth1 = []
    noti = []
    notimdr = []
    artcovg = []
    prvty = []
    prvt_mdry = []
    prvt_hivy = []
    ARTI = []
    incty = []
    inc_hivy = []
    inc_mdry = []
    mrty = []
    mrt_hivy = []
    notiy = []
    notimdry = []
    LTBI_recent = []
    LTBI_remote = []
    LTBI = []
    pop = []
    Sus1 = []
    Sus2 = []

    #% mk = round(size(xs,1)/25);
    #% for ii = 1:size(xs,1)
    #%     if mod(ii,mk) == 0; fprintf('%0.5g ',ii/mk); end 
    
    r, p = alloc_parameters(xs,prm['r'],prm['p'], xi)
    #% Get the initial conditions
    [out, aux] = obj(xs)
    init = aux['soln'][-1,:]
    
    #% Extend initial condition for the full model with vaccine compartments  
    inds1 = np.intersect1d(s0['U'], s0['v0']) #inds1 = intersect(s0.U, s0.v0);
    inds2 = np.intersect1d(np.concatenate([s0['Lf'], s0['Ls']]), s0['v0'])#inds2 = intersect([s0.Lf, s0.Ls], s0.v0);
    inds3 = np.concatenate([s0['Isc'], s0['I'], s0['E'], s0['Rlo'], s0['Rhi'], s0['R']])#inds3 = [s0.Isc, s0.I, s0.E, s0.Rlo, s0.Rhi, s0.R];
    inds4 = np.concatenate([s0['Dx'],s0['Tx'],s0['Tx2']]) #inds4 = [s0.Dx,s0.Tx,s0.Tx2];
    inds5 =  s0['Tx2'][-1]+np.array(range(9))#;  #% Auxiliary terms
    ind = np.concatenate([inds1, inds2, inds3,inds4,inds5])
   
    #% To allocate results from a model without vaccine to input vector for full model with vaccine
    init0 = np.zeros((1,i0['nx']))#init0 = zeros(1,i0.nx);
    init0[0][ind] = init #init0(ind) = init;   

    tref = range(2022,tend2+1,1) #tref = [2022:1:tend2];
     
    #% --- Now simulate without intervention (baseline)
    p['pu'] = p['pu_2022']; r['cs'] = r['cs_2022']; r['cs2'] = r['cs2_2022']# p.pu = p.pu_2022; r.cs = r.cs_2022; r.cs2 = r.cs2_2022;
    prm['growth'] = prm['gth'][3]#prm.growth = prm.gth(4);
    M0 = make_model3(p, r, i0, s0, gps0, prm); 
        
    #% trend=(1-((data.inc2015(2)-data.inc2022(2))/(data.inc2015(2)*(2022-2015))));
    #%r.betared = 0.987; %trend; % Additional reduction of beta to capture current incidence trend.
    #geq = @(t,in) goveqs_scaleup(t, in, r, M0, M0, [2022 2023], i0, prm, sel0, agg0);
    #[t0, soln0] = ode15s(geq, [2022:tend2], init0, odeset('NonNegative',[1:i0.nstates]));
    t0, soln0 = ode15s(goveqs_scaleup, init0[0], create_steps(2022, tend2, 1/12), args=(r, M0, M0, [2022, 2023],i0,prm, sel0, agg0))
       
    soln  = soln0
    t     = t0
    #soln0 = interp1(t, soln, tref);
    soln0 = np.zeros((len(tref),np.shape(soln)[1]))
    for jj in range(np.shape(soln)[1]):#range(i['nx']):
        temp_fn_sol = interpolate.interp1d(t, soln[:,jj],fill_value="extrapolate") 
        soln0[:,jj] = [temp_fn_sol(ti) for ti in tref]

     
    #% Find the initial condition at the starting point for the interventions
    #geq = @(t,in) goveqs_scaleup(t, in, r, M0, M0, [2022 2023], i0, prm, sel0, agg0);
    #[ta, solna] = ode15s(geq, [2022:tend1], init0, odeset('NonNegative',[1:i0.nstates]));
    #initb = solna(end,:);
    ta, solna = ode15s(goveqs_scaleup, init0[0], create_steps(2022, tend1, 1), args=(r, M0, M0, [2022, 2023],i0,prm, sel0,  agg0))
    initb = solna[-1,:]
   
    #% --- Next simulate with interventions
    r1 = r.copy(); p1 = p.copy()
    p1['pu']  = p['pu'] + (1- p['pu'])*0.35; 
    p1['Dx'][0] = 0.95;          #% [0.95 0.95];  
    
    #%     r1.cs(4)  = 0.8*r1.cs(4);  %1.2     %choose specific age group
    #%     r1.cs2(4) = 0.8*r1.cs2(4); %1.2     %choose specific age group
    #%     r1.cs3(4) = 0.8*r1.cs(4);  %1.2     %choose specific age group
    
    age_gp = 3 #age_gp = 4 #;  % choose specific age group
    csfac = 1.2 #; % Choose any positive number to increase or decrease of care-seeking rate 
    r1['cs'][age_gp]  = max(r['cs_1997'][age_gp],r['cs'][age_gp]*csfac)#r1.cs(age_gp)  = max(r.cs_1997(age_gp),r.cs(age_gp)*csfac);
    r1['cs2'][age_gp] = max(r['cs2_1997'][age_gp],r['cs2'][age_gp]*csfac) #r1.cs2(age_gp) = max(r.cs2_1997(age_gp),r.cs2(age_gp)*csfac);
    r1['cs3'][age_gp] = max(r['cs_1997'][age_gp],r['cs'][age_gp]*csfac) #r1.cs3(age_gp) = max(r.cs_1997(age_gp),r.cs(age_gp)*csfac);
    
    qq = 0.05;                                          # % Preventive therapy among household contacts independent of hiv status
    r1['progression'][3,:]  = r['progression'][3,:]*(1-qq) #r1.progression(4,:)  = r.progression(4,:)*(1-qq);    % Replace by this  --> p.PT_PLHIV = [1 1 (1-0.6)]
    r1['reactivation'][3,:] = r['reactivation'][3,:]*(1-qq)#r1.reactivation(4,:) = r.reactivation(4,:)*(1-qq);    
    p['PT_PLHIV'] = np.array([1, 1, (1-0.6)]) #p.PT_PLHIV = [1 1 (1-0.6)];                           % PT among PLHIV reduces rate by 60%
    
    p1['MDR_rec'] = np.array([0.5,0]) #p1.MDR_rec = [0.5,0]; %0.7; %0.8;
    r1['Tx2'] = 9/12 #r1.Tx2 = 9/12;                             % Shorter Tx2 regimn (9 months)
    p1['cure2'] = np.array([0.7, 0]) #p1.cure2 = [0.7, 0]; %0.8; %0.9;
    r1['default'][0] = r['default'][0]*(1-0.05) #r1.default(1) = r.default(1)*(1-0.05);     % reduction of default rate(pu) (by inccreasing Tx completion and reducing LTFU)     
 
    r1['ART_init'] = r['ART_init']*(1+0.5) #r1.ART_init = r.ART_init*(1+0.5);          % Increase of ART coverage
    M1 = make_model3(p1, r1, i0, s0, gps0,prm)
      
    #geq = @(t,in) goveqs_scaleup(t, in, r, M0, M1, tend1 + [0  3], i0, prm, sel0, agg0);
    #[tb, solnb] = ode15s(geq, [tend1 tend2], initb, opts); 
    tb, solnb = ode15s(goveqs_scaleup, initb, create_steps(tend1, tend2, 1/2), args=(r, M0, M1, tend1 + np.array([0,  3]),i0,prm, sel0, agg0))
   
    soln  = np.concatenate([solna, solnb[1:,:]]) #soln  = [solna; solnb(2:end,:)]; 
    t     = np.concatenate([ta,    tb[1:]]) #t     = [ta;    tb(2:end)];
     
    soln8 = np.zeros((len(tref),np.shape(soln)[1])) #soln8 = interp1(t, soln, tref); 
    for jj in range(np.shape(soln)[1]):#range(i['nx']):
        temp_fn_sol = interpolate.interp1d(t, soln[:,jj],fill_value="extrapolate") 
        soln8[:,jj] = [temp_fn_sol(ti) for ti in tref]
    
    #% --- Next simulate with vaccination
    #% Find vaccination starting point
    #[tv, solnv] = ode15s(geq, [tend1 tend1b], initb, opts);
    tv, solnv = ode15s(goveqs_scaleup, initb, create_steps(tend1, tend1b, 1/2), args=(r, M0, M1, tend1 + np.array([0,  3]),i0,prm, sel0, agg0))
    initv = solnv[-1,:] #initv = solnv(end,:);
   
    #% Choose suitable rate of vaccination and vaccine efficacy
    r2 = r1.copy(); p2 = p1.copy() #r2 = r1; p2 = p1;
    p2['VE'] = np.array([0,  0.7]) #p2.VE = [0  0.7];                    % Efficacy--> [infection prevention, disease prevention]
    r2['vacc'] = np.array([0, 0, 5, 5, 5]) #r2.vacc = [0 0 5 5 5];               % Vaccination rate (chosen)
    M2 = make_model3(p2, r2, i0, s0, gps0, prm) #M2 = make_model3(p2, r2, i0, s0, gps0, prm); % With vaccine
      
    #geq = @(t,in) goveqs_scaleup(t, in,r, M1, M2, tend1b + [0  6], i0, prm, sel0, agg0); 
    #[tc, solnc] = ode15s(geq, [tend1b tend2], initv, opts);
    tc, solnc = ode15s(goveqs_scaleup, initv, create_steps(tend1b, tend2, 1/2), args=(r, M1, M2, tend1b + np.array([0,  6]),i0,prm, sel0, agg0))
    
    soln  = np.concatenate([solna, solnv[1:,:], solnc[1:,:]]) #soln  = [solna; solnv(2:end,:); solnc(2:end,:)];
    t     = np.concatenate([ta,    tv[1:], tc[1:]]) #t     = [ta;    tv(2:end);      tc(2:end)];
     
    soln9 = np.zeros((len(tref),np.shape(soln)[1])) #soln8 = interp1(t, soln, tref); 
    for jj in range(np.shape(soln)[1]):#range(i['nx']):
        temp_fn_sol = interpolate.interp1d(t, soln[:,jj],fill_value="extrapolate") 
        soln9[:,jj] = [temp_fn_sol(ti) for ti in tref]
    
    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%     
    #% --- Bring them all together (baseline, interventions other than vaccine, interventions including vaccine)
    #allmat = cat(3, soln0,soln8,soln9); 
    allmat = np.dstack((soln0, soln8, soln9)) #allmat = cat(3, soln0,soln8,soln9); 
    dallmat = np.diff(allmat,n=1,axis=0) #dallmat = np.diff(allmat,[],1)
    
    inct = np.squeeze(dallmat[:,i0['aux']['inc'][0],:])*1e5 #inct(:,:) = squeeze(dallmat(:,i0.aux.inc(1),:))*1e5;    % Total incidence
    inch1 = np.squeeze(dallmat[:,i0['aux']['inc'][1],:])*1e5 #inch1(:,:) = squeeze(dallmat(:,i0.aux.inc(2),:))*1e5;   % HIV incidence
    mdr = np.squeeze(dallmat[:,i0['aux']['inc'][2],:])*1e5 #mdr(:,:) = squeeze(dallmat(:,i0.aux.inc(3),:))*1e5;     % MDR incidence
    morth0 = np.squeeze(dallmat[:,i0['aux']['mort'][0],:])*1e5 #morth0(:,:) = squeeze(dallmat(:,i0.aux.mort(1),:))*1e5; % hiv-negative mortality
    morth1 = np.squeeze(dallmat[:,i0['aux']['mort'][1],:])*1e5 #morth1(:,:) = squeeze(dallmat(:,i0.aux.mort(2),:))*1e5; % hiv-positive mortality
    noti = np.squeeze(np.sum(dallmat[:,i0['aux']['noti'],:],1))*1e5 #noti(:,:) = squeeze(sum(dallmat(:,i0.aux.noti,:),2))*1e5; % Total notification
    notimdr = np.squeeze(dallmat[:,i0['aux']['noti2'],:])*1e5 #notimdr(:,:) = squeeze(dallmat(:,i0.aux.noti2,:))*1e5;  % MDR Treatment initiation
   
    artcovg = np.squeeze(np.sum(allmat[:,s0['hart'],:],1)/np.sum(allmat[:,np.concatenate([s0['h1'],s0['hart']]),:],1))#artcovg(:,:) = squeeze(sum(allmat(:,s.hart,:),2)./sum(allmat(:,[s.h1,s.hart],:),2));
    
    #%   prvt = squeeze(sum(allmat(:,s0.prevalent,:),2))*1e5;       % Prevalence
    prvty = np.squeeze(np.sum(allmat[1:,s0['prevalent'],:],1)) #prvty(:,:) = squeeze(sum(allmat(2:end,s0.prevalent,:),2));                              %yearly prevalence
    prvt_mdry = np.squeeze(np.sum(allmat[1:,np.intersect1d(s0['prevalent'],s0['mdr']),:],1)) #prvt_mdry(:,:) = squeeze(sum(allmat(2:end,intersect(s0.prevalent,s0.mdr),:),2));         %yearly mdr prevalence
    prvt_hivy = np.squeeze(np.sum(allmat[1:,np.intersect1d(s0['prevalent'],[s0['h1'],s0['hart']]),:],1)) #prvt_hivy(:,:) = squeeze(sum(allmat(2:end,intersect(s0.prevalent,[s0.h1,s0.hart]),:),2)); %yearly hiv prevalence
    ARTI = np.squeeze(xs[xi['r_beta']]*np.sum(allmat[1:,np.intersect1d(s0['infectious'],s0['h0']),:],1)/np.sum(allmat[1:,np.array(range(len(d0))),:],1)) #ARTI(:,:) = squeeze(xs(xi.r_beta)*sum(allmat(2:end,intersect(s0.infectious,s.h0),:),2)./sum(allmat(2:end,1:size(d0,1),:),2));  % Yearly

    incty = np.squeeze(np.diff(allmat[:,i0['aux']['inc'][0],:],0)) #incty(:,:) = squeeze(diff(allmat(:,i0.aux.inc(1),:),[],1));      %yearly all incidence
    inc_hivy = np.squeeze(np.diff(allmat[:,i0['aux']['inc'][1],:],0)) #inc_hivy(:,:) = squeeze(diff(allmat(:,i0.aux.inc(2),:),[],1));   %yearly hiv-tb incidence
    inc_mdry = np.squeeze(np.diff(allmat[:,i0['aux']['inc'][2],:],0)) #inc_mdry(:,:) = squeeze(diff(allmat(:,i0.aux.inc(3),:),[],1));   %yearly mdr incidence
    mrty = np.squeeze(np.diff(allmat[:,i0['aux']['mort'][0],:],0)) #mrty(:,:)  = squeeze(diff(allmat(:,i0.aux.mort(1),:),[],1));     %yearly hiv-negative TB mortality
    mrt_hivy  = np.squeeze(np.diff(allmat[:,i0['aux']['mort'][1],:],0)) #mrt_hivy(:,:)  = squeeze(diff(allmat(:,i0.aux.mort(2),:),[],1)); %yearly hiv-positive TB mortality
    notiy = np.squeeze(np.sum(np.diff(allmat[:,i0['aux']['noti'],:],0),1)) #notiy(:,:) = squeeze(sum(diff(allmat(:,i0.aux.noti,:),[],1),2)); %yearly notification 
    notimdry = np.squeeze(np.diff(allmat[:,i0['aux']['noti2'],:],0)) #notimdry(:,:) = squeeze(diff(allmat(:,i0.aux.noti2,:),[],1));    %yearly MDR Tx initiation
   
    LTBI_recent = np.squeeze(np.sum(allmat[:,np.intersect1d(s0['Lf'],s0['h0']),:],1)) #LTBI_recent(:,:) = squeeze(sum(allmat(:,intersect(s0.Lf,s0.h0),:),2));   %yearly LTBI (only hiv-ve) 
    LTBI_remote = np.squeeze(np.sum(allmat[:,np.intersect1d(s0['Ls'],s0['h0']),:],1)) #LTBI_remote(:,:) = squeeze(sum(allmat(:,intersect(s0.Ls,s0.h0),:),2));   %yearly LTBI (only hiv-ve)

    LTBI = np.squeeze(np.sum(allmat[:,np.concatenate([s0['Lf'],s0['Ls']]),:],1))  #LTBI(:,:) = squeeze(sum(allmat(:,[s0.Lf,s0.Ls],:),2));         %yearly total LTBI
    pop = np.squeeze(np.sum(allmat[:,np.concatenate([s0['U'],s0['Lf'],s0['Ls'],s0['prevalent'],s0['Rlo'],s0['Rhi'],s0['R']]),:],1)) #pop(:,:) = squeeze(sum(allmat(:,[s0.U,s0.Lf,s0.Ls,s0.prevalent,s0.Rlo,s0.Rhi,s0.R],:),2)); 
    #% TB_woTx(:,:,ii) = squeeze(sum(allmat(2:12:end,s.infectious,:),2));         %yearly Active TB cases without Tx 
   
    Sus1 = np.squeeze(np.sum(allmat[:,np.intersect1d(np.concatenate([s0['U'],s0['Lf'],s0['Ls'], s0['Rlo'], s0['Rhi'], s0['R']]),s0['h0']),:],1)) #Sus1(:,:) = squeeze(sum(allmat(:,intersect([s0.U,s0.Lf,s0.Ls, s0.Rlo, s0.Rhi, s0.R],s0.h0),:),2)); % Susceptiable population including U
    Sus2 = np.squeeze(np.sum(allmat[:,np.intersect1d(np.concatenate([s0['Lf'],s0['Ls'], s0['Rlo'], s0['Rhi'], s0['R']]),s0['h0']),:],1))#Sus2(:,:) = squeeze(sum(allmat(:,intersect([s0.Lf,s0.Ls, s0.Rlo, s0.Rhi, s0.R],s0.h0),:),2));     % Susceptiable population excluding U
        
    tab = {}
    tab['pop'] = np.concatenate([np.array(data['pop00_22'])[np.array(range(len(data['pop00_22'])-3,len(data['pop00_22'])-1))], np.array([43326489, 43044540,	42760880,	42476294,	42190115,	41901694,	41610116,	41314544,	41014863])]) #tab.pop       = [data.pop00_22(end-3:end-1), 43326489, 43044540	42760880	42476294	42190115	41901694	41610116	41314544	41014863]; % pop_19_30;                                        %pop from 2019-2029
    tab['prvty_pct'] = prvty*1e5 #np.transpose(prvty*1e5,[1,0,2]) #tab.prvty_pct = permute(prvty*1e5,[2,1,3]);       %yearly prevalence
    tab['prvtmdry_pct'] = prvt_mdry*1e5 #np.transpose(prvt_mdry*1e5,[1,0,2])#tab.prvtmdry_pct = permute(prvt_mdry*1e5,[2,1,3]);%yearly mdr-TB-prevalence
    tab['prvthivy_pct'] = prvt_hivy*1e5 #np.transpose(prvt_hivy*1e5,[1,0,2]) #tab.prvthivy_pct = permute(prvt_hivy*1e5,[2,1,3]);%yearly hiv-TB-prevalence
    tab['ARTI'] = ARTI*1e2 #np.transpose(ARTI*1e2,[1,0,2]) #tab.ARTI = permute(ARTI*1e2,[2,1,3]);            % In percentage    

    tab['incy_pct']  = incty*1e5 #np.transpose(incty*1e5,[1,0,2]) #tab.incy_pct  = permute(incty*1e5,[2,1,3]);       %yearly incidence
    tab['inchivy_pct']  = inc_hivy*1e5 #np.transpose(inc_hivy*1e5,[1,0,2]) #tab.inchivy_pct  = permute(inc_hivy*1e5,[2,1,3]); %yearly hiv-TB incidence
    tab['incmdry_pct']  = inc_mdry*1e5 #np.transpose(inc_mdry*1e5,[1,0,2]) #tab.incmdry_pct  = permute(inc_mdry*1e5,[2,1,3]); %yearly mdr-incidence

    tab['mrty_pct']  = mrty*1e5 #np.transpose(mrty*1e5,[1,0,2]) #tab.mrty_pct  = permute(mrty*1e5,[2,1,3]);        %yearly hiv-ve mortality
    tab['mrthivy_pct']  = mrt_hivy*1e5 #np.transpose(mrt_hivy*1e5,[1,0,2]) #tab.mrthivy_pct  = permute(mrt_hivy*1e5,[2,1,3]); %yearly hiv-TB mortality

    tab['LTBIrecent_pct'] = LTBI_recent*1e5 #np.transpose(LTBI_recent*1e5,[1,0,2])#tab.LTBIrecent_pct = permute(LTBI_recent*1e5,[2,1,3]);  %yearly LTBI recent(fast)
    tab['LTBIremote_pct'] = LTBI_remote*1e5 #np.transpose(LTBI_remote*1e5,[1,0,2]) #tab.LTBIremote_pct = permute(LTBI_remote*1e5,[2,1,3]);  %yearly LTBI remote(slow)

    tab['LTBI_pct'] = LTBI*1e5 #np.transpose(LTBI*1e5,[1,0,2])#tab.LTBI_pct = permute(LTBI*1e5,[2,1,3]);         %yearly LTBI remote(slow)
    tab['population_pct'] = pop*1e5 #np.transpose(pop*1e5,[1,0,2])#tab.population_pct = permute(pop*1e5,[2,1,3]);    %yearly LTBI remote(slow)

    tab['noti_pct'] = notiy*1e5 #np.transpose(notiy*1e5,[1,0,2]) #tab.noti_pct = permute(notiy*1e5,[2,1,3]);        % Yearly notification
    tab['notimdry_pct'] = notimdry*1e5 #np.transpose(notimdry*1e5,[1,0,2]) #tab.notimdry_pct = permute(notimdry*1e5,[2,1,3]); %yearly mdr Tx initiation
    
    return tend1, tend1b, tend2, inct, mdr, noti, notimdr, tab
    #save res_forward; %with disruption   


tend1, tend1b, tend2, inct, mdr, noti, notimdr, tab = run_simulate_forward('NGA',None)

fig, ((ax1, ax2), (ax3, ax4)) = plt.subplots(nrows=2, ncols=2)

ax1.plot(np.array(range(2022,tend2)),inct)
ax1.set_xlabel('Time (year)')
ax1.set_ylabel('Incidence')

ax2.plot(np.array(range(2022,tend2)),mdr)
ax2.set_xlabel('Time (year)')
ax2.set_ylabel('MDR incidence')

ax3.plot(np.array(range(2022,tend2)),noti)
ax3.set_xlabel('Time (year)')
ax3.set_ylabel('Notifications')

ax4.plot(np.array(range(2022,tend2)),notimdr)
ax4.set_xlabel('Time (year)')
ax4.set_ylabel('MDR treatment initiation')

fig.show()
a=0
#%  Figure_final
 
#figure; lw = 1.5;
#subplot(2,2,1)
#plot(2022:tend2-1,inct,'linewidth',lw) %inct
#ylabel('All Incidence')
#subplot(2,2,2)
#plot(2022:tend2-1,mdr,'linewidth',lw)
#ylabel('MDR incidence')
#subplot(2,2,3)
#plot(2022:tend2-1,noti,'linewidth',lw)
#ylabel('Notification')
#subplot(2,2,4)
#pl1 = plot(2022:tend2-1,notimdr,'linewidth',lw);
#ylabel('MDR treatment initiation')
#%legend(pl1,'Baseline','+PPM','+ Improved diagnostics, routine TB services','+ Upstream case-finding','+ case-finding among subclinical','+ Preventive therapy (risk group)','+ Upfront DST','+ Shorter duration of 2ndline Tx','+ Improved secondline Tx success','+ Vaccination','location','SouthEast'); %
#legend(pl1,'Baseline','Interventions without vaccine','+ Vaccination','location','SouthEast'); %
 





import numpy as np
#import os
#import MapTPstoPINs
#import ReadTargetPopAndIntvnData
#import CalcResourceNeeds
# import InterventionInit

from .RunGlobalEpiModelb import RunGlobalEpiModelb
import pickle
import json
import math
from .get_distribution_fns import get_distribution_fns
from scipy import interpolate

#from InterventionScenario import InterventionScenario
from .InterventionInit import InterventionInit

from .get_objective import get_objective as f_obj
from loguru import logger

import os

def run_dynamical_model(ISO3, prm_modvar, xs_modvar, pop_data_modvar, Hi_scn, Hi_ref, TxTSFL_scn, TxTSFL_ref, Hd_scn, Hd_ref, firstImpactYr, finalImpactYr, finalYr, final_model_path=''):
    logger.debug('Starting run_dynamical_model')
    Model_setup_file_name = f"{final_model_path}/data/SetupModel/Model_setup.pkl"
    init0 = []
    Model_init0_file_name = f"{final_model_path}/data/json_init/" + ISO3 + "_init0.json"
    if os.path.exists(Model_init0_file_name):
         if os.path.isfile(Model_init0_file_name):
              with open(Model_init0_file_name , "r") as fp:#with open(f"{final_model_path}\\data\\CtriesDefaults\\TB.JSON", "r") as fp:
                init0 = json.load(fp)['init0']
                 
    with open(Model_setup_file_name, 'rb') as fp:
        setup_data = pickle.load(fp)
    logger.debug(f'Hi_scn {Hi_scn}')
    logger.debug(f'Hi_ref {Hi_ref}')
    logger.debug(f'TxTSFL_scn {TxTSFL_scn}')
    logger.debug(f'TxTSFL_ref {TxTSFL_ref}')
    logger.debug(f'Hd_scn {Hd_scn}')
    logger.debug(f'Hd_ref {Hd_ref}')


    ScnPARS = [ISO3]
    #ScnPARS{1}=ISO3;

    #close all;%figures
    #clear all;%variables 

    #%Location of Dynamical model 
    #cd C:\work\GlobalModel\NGA_Final

    #%these two lines can be commented out in the initial Python translation
    #load calibres1.mat xsto2 outsto2 xi obj; 
    #load Model_setup.mat i0 s0 gps0 sel0 agg0 r p prm d s;
    #######################%%%%mat_file_name = f"{final_model_path}/data/Calibres/" + ISO3 + ".mat"
    # jso_file_name = f"{final_model_path}/data/CtriesDefaults/" + ISO3 + "_calibres.json"

    #################mat_contents = scipy.io.loadmat(mat_file_name) #mat_contents = scipy.io.loadmat('calibres1.mat')
    mat_contents = {}

    # if data_glob_input==None:
    #     f_glob_input = open('TBExample.JSON')
    #     data_glob_input = json.load(f_glob_input)


    
    #Loading NGA specific data
    # if prm_modvar:
    data = pop_data_modvar.copy()
    prm = prm_modvar.copy()

    if len(prm['r']['ageing'])==5 :
        prm['r']['ageing'] = prm['r']['ageing'][1:]

    r = prm['r'].copy()
    p = prm['p'].copy()
    xs = xs_modvar
    # else:
    #     f_data_ctr = open(jso_file_name)
    #     data_ctr = json.load(f_data_ctr)
    #     data = data_ctr['data']
    #     prm = data_ctr['prm']
    #     r = data_ctr['r']
    #     p = data_ctr['p']
    #     xs = data_ctr['xs']
    
    #cs_global = r['cs']
    #cs2_global = r['cs2']
    #pu_global = p['pu']

    #% Age-group of Population in each compartment [0-4,5-9,10-14,15-65,65+]   
    #prm['N'] = np.array([0.2, 0.2, 0.2, 0.2, 0.2]) #prm.N = [0.2 0.2 0.2 0.2 0.2]; % Initial population (assumption)
    #prm['cm'] = data['contact_mat'] #prm.cm = data.contact_mat;     % Contact matrix correcting with population weight
    #prm['gth'] = np.array([0, 0.03, 0.035, 0.044]) #prm.gth = [0 0.03 0.035 0.044]; %[0,data.gth];        % population growth rate
    #r['mort'] = data['age_mort'] #r.mort = data.age_mort';        % Age specific mortality rate
    #r['ageing'] = np.array([0.1774, 0.1696, 0.1594, 0.0050]) #r.ageing = [0.1774 0.1696 0.1594 0.0050]; %data.ageing';   % Ageing rate

    # self cure is missing from r. I am adding it manually. Carel should make sure that it is included in the country specific JSON file0.1667        0   0.1667 
    #r['self_cure'] = np.array([0.1667,        0,   0.1667])


    #% --- Get calibration targets ---------------------------------------------
    #%data.pop00_22 = data.pop00_22;
    popn          = data['pop00_22'][-1] #popn          = data.pop00_22(end); % 2022
    logger.debug(f"popn {popn}")
    logger.debug(f"HIV_prev before {np.array(data['HIV_prev'])}")
    data['inc2015']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-8,:] #data.inc2015  = data.inc_all(end-7,:); 
    data['inc2022']  = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-1,:] #data.inc2022  = data.inc_all(end,:); 
    data['inc2000'] = np.array(data['inc_all'])[np.array(data['inc_all']).shape[0]-23,:]
    #% data.inc_h1  = data.inc_h1;
    #% data.mdr2015  = data.mdr2015; 
    #% data.mdr2022  = data.mdr2022;
    #data['mort_H0']  = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-1,:] #data.mort_H0  = data.mort_H0(end,:);
    data['mort_H02000'] = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-23,:]
    data['mort_H02022'] = np.array(data['mort_H0'])[np.array(data['mort_H0']).shape[0]-1,:]
    #%data.mort_H1  = data.mort_H1;
    data['mort_H12022'] = np.array(data['mort_H1'])
    data['noti'] = np.array([0.8, 1, 1.2])*np.array(data['noti'])[-1] #data.noti     = [0.8 1 1.2]*data.noti;
    data['mdriniTX'] = np.array([0.8, 1, 1.2])*np.array(data['mdriniTX'])[-1]  #data.mdriniTX = [0.8 1 1.2]*data.mdriniTX; 
    #data['sym']      = np.array([0.36, 0.50, 0.80])#data.sym      = [0.36 0.50 0.80];        % proportion of prevalent TB that has symptoms

    #% Get the HIV data                                                         
    prm['ART_start'] = data['ART_start'] #prm.ART_start = data.ART_start;                     %ART_start;
    data['ART_covg'] =  np.array([0.85, 1, 1.15])*data['ART_covg'] #data.ART_covg =  [0.85 1 1.15]*data.ART_covg;       %ARTcovg_2022;
    data['HIV_prev'] = np.array([0.85, 1, 1.15])*(np.array(data['HIV_prev'])/popn) #data.HIV_prev = [0.85 1 1.15]*(data.HIV_prev./popn);%HIVprev_2022 (proportion of population)
    HIV_incd = data['HIV_incd'] #HIV_incd = data.HIV_incd;                           %HIV incidence (proportion of population) 1980-2022

    logger.debug(f"HIV_prev {data['HIV_prev']} ")
    logger.debug(f"ART_covg {data['ART_covg']} ")
    logger.debug(f"ART_start {data['ART_start']} ")
    logger.debug(f"HIV_incd {data['HIV_incd']} ")

    logger.debug(f"noti {data['noti']}")
    logger.debug(f"mort_H12022 {data['mort_H12022']}")

    #% Extrapolate HIV incidence by 20 years
    ys1 = HIV_incd #;   %1980-2022
    xs1 = np.array(range(len(ys1))) #xs1 = 1:length(ys1);
    xs2 = np.array(range(len(ys1)))+20#xs2 = 1:length(ys1)+20;    % 2022-2042
    #ys2 = interp1(xs1,ys1,xs2,'linear','extrap');  % linier trend for the next 21 years
    fnys2 = interpolate.interp1d(xs1,ys1,kind='linear',fill_value='extrapolate')
    ys2 = [fnys2(o) for o in xs2]


    #GUY: I am commenting this out because Matlab uses something different 
    #prm['rHIV'] = ys2 #prm.rHIV = ys2;
    #prm['r'] = r #@Carel: is this OK? The r in prm does not have the expected structure
    #prm['r']['self_cure'] = np.array([0.1667,        0,   0.1667])
    #prm['N'] = np.array([0.2, 0.2, 0.2, 0.2, 0.2]) #prm.N = [0.2 0.2 0.2 0.2 0.2]; % Initial population (assumption)
    #prm['cm'] = data['contact_mat'] #prm.cm = data.contact_mat;     % Contact matrix correcting with population weight
    #prm['gth'] = np.array([0, 0.03, 0.035, 0.044]) #prm.gth = [0 0.03 0.035 0.044]; %[0,data.gth];        % population growth rate
    #prm['r']['ageing'] = np.array([0.1774, 0.1696, 0.1594, 0.0050]) #r.ageing = [0.1774 0.1696 0.1594 0.0050]; %data.ageing';   % Ageing rate

    #prm['r']['cs'] =  cs_global
    #prm['r']['cs2'] =  cs2_global
    #prm['p']['pu'] = pu_global
    #r = prm['r']
    #END GUY: I am commenting this out because Matlab uses something different 

    show = 0
    f0 = get_distribution_fns(data['sym'],   'lognorm', show) #f0 = get_distribution_fns(data.sym,         'lognorm', show);#f0 = get_distribution_fns(data.inc2015,   'lognorm', show)
    
    #I add this here so that the code can run, Carel should update 'data' in the JSON file 
    #data['inc2000'] = data['inc2022'] 
    #data['mort_H02000'] = data['mort_H0']
    #data['mort_H02022'] = data['mort_H0']
    #data['noti_cu'] = data['noti']
    data['noti2022'] = data['noti']*np.array([0.8, 1, 1.2])
    data['noti_cu'] = np.sum(np.array(data['c_newinc'][-1]))*np.array([0.8, 1, 1.2]) #data['noti_cu'] = sum(data['c_newinc'])*np.array([0.8, 1, 1.2])
    logger.debug(f"noti2022 {data['noti2022']}")
    logger.debug(f"noti_cu {data['noti_cu']}")
    #data['mort_H12022'] = data['mort_H1']

    f1 = get_distribution_fns(data['inc2000'],     'lognorm', show) #f1 = get_distribution_fns(data.inc2000,     'lognorm', show);  
    f2 = get_distribution_fns(data['inc2022'],     'lognorm', show) #f2 = get_distribution_fns(data.inc2022,     'lognorm', show);
    f3 = get_distribution_fns(data['mort_H02000'], 'lognorm', show) #f3 = get_distribution_fns(data.mort_H02000, 'lognorm', show);
    f4 = get_distribution_fns(data['mort_H02022'], 'lognorm', show)#f4 = get_distribution_fns(data.mort_H02022, 'lognorm', show);
    f5 = get_distribution_fns(data['noti_cu'],     'lognorm', show)#f5 = get_distribution_fns(data.noti_cu,     'lognorm', show);
    f6 = get_distribution_fns(data['noti2022'],    'lognorm', show)#f6 = get_distribution_fns(data.noti2022,    'lognorm', show);
    f7 = get_distribution_fns(data['mdr2015'],     'lognorm', show)#f7 = get_distribution_fns(data.mdr2015,     'lognorm', show);
    f8 = get_distribution_fns(data['mdr2022'],     'lognorm', show)#f8 = get_distribution_fns(data.mdr2022,     'lognorm', show);
    f9 = get_distribution_fns(data['mdriniTX'],    'lognorm', show)#f9 = get_distribution_fns(data.mdriniTX,    'lognorm', show); 
    f10 = get_distribution_fns(data['HIV_prev'],   'lognorm', show)#f10 = get_distribution_fns(data.HIV_prev,   'lognorm', show);
    f11 = get_distribution_fns(data['inc_h1'],     'lognorm', show) #f11 = get_distribution_fns(data.inc_h1,     'lognorm', show);
    f12 = get_distribution_fns(data['ART_covg'],   'lognorm', show)#f12 = get_distribution_fns(data.ART_covg,   'lognorm', show);
    f13 = get_distribution_fns(data['mort_H12022'],'lognorm', show)#f13 = get_distribution_fns(data.mort_H12022,'lognorm', show);

    logger.debug("after get_distribution_fns")
    #lhd.fn  = @(sym,inc2000, inc2022, mort_H02000,mort_H02022,noti_cu, noti2022, mdr2015, mdr2022, mdriniTX,HIV_prev,inc_h1,ART_covg, mort_H12022)f0(sym)+f1(inc2000)+ 10*f2(inc2022)+f3(mort_H02000)+ 10*f4(mort_H02022)+ f5(noti_cu)+f6(noti2022)+f7(mdr2015)+f8(mdr2022)+f9(mdriniTX)+f10(HIV_prev)+f11(inc_h1)+f12(ART_covg)+f13(mort_H12022);
    def fn(sym,inc2000, inc2022, mort_H02000,mort_H02022,noti_cu, noti2022, mdr2015, mdr2022, mdriniTX,HIV_prev,inc_h1,ART_covg, mort_H12022):   
            res_0 = f0[0](sym)
            res_1 = f1[0](inc2000) 
            res_2 = 10*f2[0](inc2022)
            res_3 = f3[0](mort_H02000) 
            res_4 = 10*f4[0](mort_H02022) 
            res_5 = f5[0](noti_cu)
            res_6 = f6[0](noti2022)
            res_7 = f7[0](mdr2015)
            res_8 = f8[0](mdr2022)
            res_9 = f9[0](mdriniTX)
            res_10 = f10[0](HIV_prev)
            res_11 = f11[0](inc_h1)
            res_12 = f12[0](ART_covg)
            res_13 = f13[0](mort_H12022)
            result = res_0 + res_1 + res_2 + res_3 + res_4 + res_5 + res_6 + res_7 + res_8 + res_9 + res_10 + res_11 + res_12 + res_13
            return result


    lhd = {}
    lhd['fn']  = fn  
    lhd['sgn'] = -math.inf

    logger.debug("defining obj")
    def obj(): #def obj(x): # The signature was changed on June 09 2024 after Carel decided that we stop passing xs as a parameter
        #[r_out, r_aux] = f_obj(x, prm.copy(), setup_data['ref'], setup_data['sel'].copy(), setup_data['agg'].copy(), setup_data['gps'].copy(), setup_data['gps1'].copy(), setup_data['gps2'].copy(),lhd)
        [r_out, r_aux] = f_obj(prm.copy(), setup_data['ref'], setup_data['sel'].copy(), setup_data['agg'].copy(), setup_data['gps'].copy(), setup_data['gps1'].copy(), setup_data['gps2'].copy(),lhd)
        return([r_out,r_aux])

    #mat_contents['xsto2'] = np.array([xs])
    #mat_contents['outsto2'] = np.array([0])
    #PARS = [mat_contents['xsto2'],mat_contents['outsto2'],setup_data['xi'],obj,setup_data['i0'],setup_data['s0'],setup_data['gps0'],
    #        setup_data['sel0'],setup_data['agg0'],r.copy(),data_glob_input['<TB_DynamicalModelPRM_V1>']['p'],
    #    prm]
    PARS = [[0],[0],setup_data['xi'],obj,setup_data['i0'],setup_data['s0'],setup_data['gps0'],
            setup_data['sel0'],setup_data['agg0'],r.copy(),p.copy(), prm.copy(), init0.copy() ]


    #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    #%Empty intervention object
    EpiIntvn=InterventionInit()
    
    EpiIntvn['year1'] = np.array([firstImpactYr])
    EpiIntvn['year2'] = np.array([finalImpactYr])
    EpiIntvn['year3'] = np.array([finalYr+1])
    

    #[epi_outputs_base]=RunGlobalEpiModelb(PARS,EpiIntvn,PathToModel,PathToScenarioModel)
    
    # EpiIntvn empty and generates no impact
    logger.debug(f'EpiIntvn base {EpiIntvn}')
    epi_outputs_base =RunGlobalEpiModelb(PARS, EpiIntvn, True) #The paths are not parameters in the version of the Matlab procedures I have access to 

    logger.debug(f'epi_outputs_base {epi_outputs_base}')
    # with open('epi_outputs_base.JSON', 'r') as fP:
    #     tmp = ujson.load(fP)
    #     epi_outputs_base = [np.array(base) for base in tmp]
    inct_base=epi_outputs_base[0] #inct_base=epi_outputs_base{1};%.*DPProjBase(4:12)'/10^5;
    inch1_base=epi_outputs_base[1] #inch1_base=epi_outputs_base{2};%.*DPProjBase(4:12)'/10^5;
    mdr_base=epi_outputs_base[2] #mdr_base=epi_outputs_base{3};%.*DPProjBase(4:12)'/10^5;
    morth0_base=epi_outputs_base[3] #morth0_base=epi_outputs_base{4};%.*DPProjBase(4:12)'/10^5;
    morth1_base=epi_outputs_base[4] #morth1_base=epi_outputs_base{5};%.*DPProjBase(4:12)'/10^5;
    mort_base=morth0_base+morth1_base#mort_base=morth0_base+morth1_base;
    noti_base=epi_outputs_base[5] #noti_base=epi_outputs_base{6};%.*DPProjBase(4:12)'/10^5;
    notimdr_base=epi_outputs_base[6] #notimdr_base=epi_outputs_base{7};%.*DPProjBase(4:12)'/10^5;
    prevtb_base=epi_outputs_base[7]

    # EpiIntvn['year1'] = np.array([2023])
    # EpiIntvn['year2'] = np.array([2026])
    # EpiIntvn['year3'] = np.array([2031])

    EpiIntvn['year1_vac'] = np.array([firstImpactYr])
    EpiIntvn['year2_vac'] = np.array([finalImpactYr])
    EpiIntvn['year3_vac'] = np.array([finalYr+1])

    #%TB detection
    EpiIntvn['r_cs'] = np.ones(5) ## EpiIntvn.r_cs=ones(1,5);          
    EpiIntvn['r_cs2'] = np.ones(5)# EpiIntvn.r_cs2=ones(1,5);         
    #%EpiIntvn.r_cs3=ones(1,5);

    #%Impact on detection
    Hd_ratio = Hd_scn/Hd_ref #Hd_ratio=Hd_scn./Hd_ref; 
    logger.debug(f'Hd_ratio {Hd_ratio}')
    

    for a in range(5) : #for a=1:5
        EpiIntvn['r_cs'][a] = Hd_ratio  
        EpiIntvn['r_cs2'][a] = Hd_ratio#EiIntvn.r_cs2(a)=Hd_ratio;
    # end
   
    #%Prevention
    EpiIntvn['r_progression'] = np.zeros((5,3)) #IntvnScn.r_progression=zeros(5,3);
    EpiIntvn['r_reactivation'] = np.zeros((5,3)) #IntvnScn.r_reactivation=zeros(5,3);

    #%Impact on infection
    Hi_ratio=(1-Hi_scn)/(1-Hi_ref) #;
    logger.debug(f'Hi_ratio {Hi_ratio}')

    EpiIntvn['r_progression'][:,0] = 1-Hi_ratio #IntvnScn.r_progression(:,1)=1-Hi_ratio;	
    EpiIntvn['r_reactivation'][:,0]= 1-Hi_ratio #IntvnScn.r_reactivation(:,1)=1-Hi_ratio;
   
    #%Treatment Success
    #Impact on treatment success
    TxTSFL_ratio=(1-TxTSFL_scn)/(1-TxTSFL_ref)#;
    logger.debug(f'TxTSFL_ratio {TxTSFL_ratio}')

    EpiIntvn['p_cure'] = np.array([TxTSFL_ratio])
    EpiIntvn['p_cure2'] = np.array([TxTSFL_ratio])

    #%SL treatment duration, only if treatment duration is set
    if len(EpiIntvn['p_cure2'])!=0:
        EpiIntvn['r_Tx2'] = np.array([12/9]) #IntvnScn.r_Tx2 = 12/9; % Shorter Tx2 regimen (9 months)

    #[epi_outputs_scn]=RunGlobalEpiModelb(PARS,EpiIntvn,PathToModel,PathToScenarioModel)
    # EpiIntvn not empty and generates impact
    epi_outputs_scn =RunGlobalEpiModelb(PARS,EpiIntvn,False) #The paths are not parameters in the version of the Matlab procedures I have access to 
    logger.debug(f'epi_outputs_scn {epi_outputs_scn}')
    # with open('epi_outputs_scn.JSON', 'r') as fP:
    #     tmp = ujson.load(fP)
    #     epi_outputs_scn = [np.array(scn) for scn in tmp]
    inct_scn=epi_outputs_scn[0] #inct_base=epi_outputs_base{1};%.*DPProjBase(4:12)'/10^5;
    inch1_scn=epi_outputs_scn[1] #inch1_base=epi_outputs_base{2};%.*DPProjBase(4:12)'/10^5;
    mdr_scn=epi_outputs_scn[2] #mdr_base=epi_outputs_base{3};%.*DPProjBase(4:12)'/10^5;
    morth0_scn=epi_outputs_scn[3] #morth0_base=epi_outputs_base{4};%.*DPProjBase(4:12)'/10^5;
    morth1_scn=epi_outputs_scn[4] #morth1_base=epi_outputs_base{5};%.*DPProjBase(4:12)'/10^5;
    mort_scn=morth0_scn+morth1_scn#mort_base=morth0_base+morth1_base;
    noti_scn=epi_outputs_scn[5] #noti_base=epi_outputs_base{6};%.*DPProjBase(4:12)'/10^5;
    notimdr_scn=epi_outputs_scn[6] #notimdr_base=epi_outputs_base{7};%.*DPProjBase(4:12)'/10^5;
    prevtb_scn=epi_outputs_scn[7]

    Outputs_Scn = []
    Outputs_Scn.append(epi_outputs_base) 
    Outputs_Scn.append(epi_outputs_scn) 
    return Outputs_Scn 
    
     
    #end





from datetime import date
import numpy as np
import math as math

import AvenirCommon.Util as acu

import SpectrumCommon.Const.GB as gbc
import SpectrumCommon.Const.PJ.PJNTags as pjmvt
import SpectrumCommon.Modvars.GB.GBUtil as gbu

import SpectrumCommon.Const.IC.ICConstLink as iccl
import SpectrumCommon.Const.IC.ICConst as icc
import SpectrumCommon.Const.IC.ICTags as icmvt
from SpectrumCommon.Const.IC.ICInterventionIDs import *
from SpectrumCommon.Const.IC.IC_TI_InterventionIDs import *
import SpectrumCommon.Modvars.IC.ICModVarUtil as icmvu
import SpectrumCommon.Modvars.IC.ICModVarDefs as icmvd
import SpectrumCommon.Modvars.IC.ICReadDatabases as icrd
import SpectrumCommon.Util.IC.ICUtilLink as icul
import SpectrumCommon.Util.IC.ICUtil as icu

def init_modvars(country_ID, first_year, final_year, extra):
    '''
        Initializes Intervention Costing MVs for new IHT projections.

            Parameters:
                countryID (string):  Ex: BEN for Benin.

                first_year (int): First year selected by user.

                final_year (int): Final year selected by user. 

                extra (dict): Any other fields.  
 
            Returns:
                mod_vars_rec (dict): MVs.
    '''

    # Extra data from required modules
    prog_area_records_mv = extra[iccl.IC_IH_PROG_AREA_RECORDS_TAG_V1]
    staff_type_records_mv = extra[iccl.IC_HW_STAFF_TYPE_RECORDS_TAG_V1]
    costing_sect_records_mv = extra[iccl.IC_PC_COSTING_SECT_RECORDS_TAG_V1]
    IHT_costing_on = extra[gbc.GB_IHT_ON] if gbc.GB_IHT_ON in extra else False
    TB_costing_on = extra[gbc.GB_TB_COSTING_ON] if gbc.GB_TB_COSTING_ON in extra else False
    LiST_costing_on = extra[gbc.GB_LIST_COSTING_ON] if gbc.GB_LIST_COSTING_ON in extra else False
    plan_type = extra[gbc.GB_Extra_plantype] if gbc.GB_Extra_plantype in extra else iccl.IC_IH_COMPREHENSIVE_PLAN

    costing_mode_mod_vars = {
        pjmvt.GB_IHTOnTag       : IHT_costing_on,
        pjmvt.GB_TBCostingOnTag : TB_costing_on,
        pjmvt.GB_LCOnTag        : LiST_costing_on,
    }
    
    # Extra data from optional modules
    active_modules = extra[gbc.GB_MODULES]
    active_methods_FP = None
    if gbc.GB_FP in active_modules:
        active_methods_FP = extra[iccl.IC_FP_ActiveMethodsTag]
    
    syph_among_WRA_mv = None
    selected_intervs_CS = None
    CS_IV_info = None
    # enter_BF_improve_data = None

    if gbc.GB_CS in active_modules:
        syph_among_WRA_mv = extra[iccl.IC_CS_SyphilisAmongWRATag]
        selected_intervs_CS = extra[iccl.IC_CS_SelectedIVSetTag]
        CS_IV_info = extra[iccl.IC_CS_IVInfoTag]
        # enter_BF_improve_data = extra[iccl.IC_CS_EnterBFImproveDataTag]

    # file_list = GB_file_list(environment, gbc.GB_IC_CONTAINER, '')
    # for i, file_name in enumerate(file_list):
    #     log(file_name)

    num_years_inputs = final_year - first_year + 1 
    num_years_outputs = final_year - first_year + 1

    num_prog_areas = icul.IC_get_IH_num_prog_areas(prog_area_records_mv)
    num_staff_types = icul.IC_get_HW_num_staff_types(staff_type_records_mv)

    costing_mode = gbu.get_costing_mode(costing_mode_mod_vars)

    num_costed_deliv_chans = 0
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        num_costed_deliv_chans = iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED_CS
    
    else:
        num_costed_deliv_chans = iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS_COSTED

    num_deliv_chan_IC = num_costed_deliv_chans + icc.IC_NUM_DELIV_CHANS

    num_costing_sections_CS = 0
    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        num_costing_sections_CS = icul.IC_get_PC_num_costing_sects(costing_sect_records_mv, iccl.IC_IH_MODULE_MST_ID, str(gbc.GB_CS))

    #######################################################################################################
    #                                                                                                     #
    #                                       Inputs                                                        #
    #                                                                                                     #
    #######################################################################################################

    drug_supply_DB = icrd.IC_load_drug_supply_DB()
    intervention_DB, intervention_DB_IHT_ID_info_only = icrd.IC_load_intervention_DB(costing_mode)
    PIN_DB = icrd.IC_load_PIN_DB(country_ID)
    PIN_by_year_DB = icrd.IC_load_PIN_by_year_DB(country_ID)
    non_impact_coverage_DB = icrd.IC_load_non_impact_coverage_DB(country_ID)
    #non_impact_coverage_by_year_DB = icrd.IC_load_non_impact_coverage_by_year_DB()
    targ_pop_direct_entry_DB = icrd.IC_load_targ_pop_direct_entry_DB(country_ID)
    targ_pop_DB = icrd.IC_load_targ_pop_DB()
    treatment_input_group_DB = icrd.IC_load_treatment_input_group_DB()
    treatment_input_DB = icrd.IC_load_treatment_input_DB()

    health_system_costs_DB_CS = None
    outpat_visit_inpat_day_costs_CS_DB = None

    if costing_mode == gbc.GB_LIST_COSTING_MODE:
        health_system_costs_DB_CS = icrd.IC_load_health_system_costs_CS_DB(country_ID)
        outpat_visit_inpat_day_costs_CS_DB = icrd.IC_load_outpat_visit_inpat_day_costs_CS_DB(country_ID)

    elif costing_mode == gbc.GB_TB_COSTING_MODE:
        outpat_visit_inpat_day_costs_TB_DB = icrd.IC_load_outpat_visit_inpat_day_costs_TB_DB(country_ID)

    #######################################################################################################
    #                                                                                                     #
    #               Pre-results / Calculated - These databases are made in ICProj                         #
    #                                                                                                     #
    #######################################################################################################

    drug_cost_per_avg_case_DB = icrd.IC_load_drug_cost_per_avg_case_DB(costing_mode)
    supply_cost_per_avg_case_DB = icrd.IC_load_supply_cost_per_avg_case_DB(costing_mode)
    med_personnel_min_per_avg_case_DB = icrd.IC_load_med_personnel_min_per_avg_case_DB(costing_mode)

    # IC_TREATMENT_INPUT_DB_NAME             = 'TreatmentInputDB'
    # IC_ADOLESCENT_HEALTH_PIN_DB_NAME       = 'AdolescentHealthPIN_DB'
    # IC_ADOLESCENT_HEALTH_COV_DB_NAME       = 'AdolescentHealthCov_DB'
    # IC_OTHER_COSTS_DB_NAME                 = 'OtherCostsDB'

    value_bool_False = False
    value_bool_True = True
    value_int = 0
    value_flt = 0.0
    value_date = date.today().strftime("%B %d, %Y"),   
    value_list = []

    #######################################################################################################
    #                                                                                                     #
    #                                       Inputs                                                        #
    #                                                                                                     #
    #######################################################################################################

    drug_supply_records = icrd.IC_init_drug_supply_records(drug_supply_DB, num_years_inputs, costing_mode, plan_type)
    targ_pop_records = icrd.IC_init_targ_pop_def_records(targ_pop_DB, costing_mode)

    interv_records = icrd.IC_init_interv_records(intervention_DB, targ_pop_records, active_modules, num_years_inputs, costing_mode)
    interv_records = icrd.IC_set_intervs_PIN(interv_records, PIN_DB, num_years_inputs)
    interv_records = icrd.IC_set_intervs_PIN_by_year(interv_records, PIN_by_year_DB, num_years_inputs)
    interv_records = icrd.IC_set_intervs_non_impact_coverage(interv_records, non_impact_coverage_DB, num_years_inputs, costing_mode)
    # There are currently no more interventions to be read in from non_impact_coverage_by_year_DB. Leave as placeholder in case this type comes back.
    #interv_records = icrd.IC_set_interv_non_impact_coverage_by_year(interv_records, non_impact_coverage_by_year_DB, num_years_inputs)
    interv_records = icrd.IC_set_intervs_targ_pop_direct_entry(interv_records, targ_pop_direct_entry_DB)
    
    # Turn off some interventions by default.
    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        # These overlap with a version from 0-14 years.
        icmvu.set_IC_interv_active(icmvu.get_IC_interv_rec(interv_records, IC_IV_ScreenProvInitHHChildren0t4_MST_ID), False)
        icmvu.set_IC_interv_active(icmvu.get_IC_interv_rec(interv_records, IC_IV_ScreenProvInitHHChildren5t14_MST_ID), False)
        
    elif costing_mode == gbc.GB_TB_COSTING_MODE:
        # These overlap with a version from 0-14 years.
        icmvu.set_IC_interv_active(icmvu.get_IC_interv_rec(interv_records, IC_TI_IV_ScreenProvInitHHChildren0t4_MST_ID), False)
        icmvu.set_IC_interv_active(icmvu.get_IC_interv_rec(interv_records, IC_TI_IV_ScreenProvInitHHChildren5t14_MST_ID), False)

    # IHT now shows all LiST interventions except the FamPlan and HIV ones, so activate them.
    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        if gbc.GB_CS in active_modules:
            required_mod_vars = {
                iccl.IC_CS_IVInfoTag             : CS_IV_info, 
                iccl.IC_CS_SelectedIVSetTag      : selected_intervs_CS,
            }

            interv_to_exclude = []

            for mstID in acu.GBRange(iccl.CS_FirstFPMstID, iccl.CS_LastFPMstID):
                interv_to_exclude.append(mstID)
            
            interv_to_exclude.append(iccl.CS_FirstAMMstID)  
            interv_to_exclude.append(iccl.CS_MstPercOnART)
            interv_to_exclude.append(int(iccl.CS_IV_PMTCT_MST_ID))
            interv_to_exclude.append(int(iccl.CS_IV_Cotrimoxazole_MST_ID))

            icul.IC_CS_set_interventions_active(required_mod_vars, interv_to_exclude)

    if costing_mode == gbc.GB_IHT_COSTING_MODE:
        if gbc.GB_FP in active_modules:
            icu.set_IC_method_intervention_active_status({
                icmvt.IC_INTERV_RECORDS_TAG : interv_records,
                iccl.IC_FP_ActiveMethodsTag    : active_methods_FP,
                pjmvt.GB_LCOnTag               : LiST_costing_on,
            })

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        icu.set_IC_CS_intervention_active_status({
            icmvt.IC_INTERV_RECORDS_TAG    : interv_records,
            iccl.IC_CS_SelectedIVSetTag    : selected_intervs_CS,
            iccl.IC_FP_ActiveMethodsTag    : active_methods_FP,
            pjmvt.GB_LCOnTag               : LiST_costing_on,
            iccl.IC_CS_IVInfoTag           : CS_IV_info,
        })

    TIG_drug_supply_records, TIG_med_personnel_records, TIG_outpat_inpat_records = \
        icrd.IC_init_treatment_input_group_records(interv_records, treatment_input_group_DB)
    TI_drug_supply_records, TI_med_personnel_records, TI_outpat_inpat_records = \
        icrd.IC_init_treatment_input_records(interv_records, treatment_input_DB, num_years_inputs)
    
    # These PINs are kept outside of the intervention records where the other PINs are kept because they will only ever get used for
    # a couple of interventions out of hundreds and we don't want to bloat the intervention records with unnecessary fields
    alt_PIN_age_group_set_1_records = icrd.IC_init_alt_PIN_age_group_set_1_records(interv_records, num_years_inputs)
    alt_PIN_age_group_set_2_records = icrd.IC_init_alt_PIN_age_group_set_2_records(interv_records, num_years_inputs)

    # If TB costing is on, we'll need access to the IHT interventions well, because the user is allowed to move IHT interventions into TB. 
    outpat_visit_inpat_day_costs_TB = []
    interv_records_IHT_ID_info_only = []

    if costing_mode == gbc.GB_TB_COSTING_MODE:
        outpat_visit_inpat_day_costs_TB = icrd.IC_set_outpat_visit_inpat_day_costs_TB_MVs(outpat_visit_inpat_day_costs_TB_DB)
        interv_records_IHT_ID_info_only = icrd.IC_init_interv_records_ID_info_only(intervention_DB_IHT_ID_info_only, str(iccl.IH_TB_MST_ID))

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:
        other_recurrent_costs, capital_costs = icrd.IC_set_outpat_visit_inpat_day_costs_CS_MVs(outpat_visit_inpat_day_costs_CS_DB)
        supply_chain_cost_perc_commodity_cost, infrastructure_investment, ratio_other_health_system_to_interv_costs, \
            income_level, supply_chain_status = icrd.IC_set_health_system_costs_MVs(health_system_costs_DB_CS, num_years_inputs) 
        
    required_mod_vars = {
        pjmvt.PJN_FirstYearTag                       : first_year, 
        pjmvt.PJN_FinalYearTag                       : final_year, 
        icmvt.IC_TREAT_INPUT_DRUG_SUPPLY_RECORDS_TAG : TI_drug_supply_records,
        iccl.IC_CS_SyphilisAmongWRATag               : syph_among_WRA_mv,
    }

    if gbc.GB_CS in active_modules:
        icu.IC_set_treatment_input_defaults_from_LiST(required_mod_vars)

    num_intervs = icmvu.get_IC_num_intervs(interv_records)
    num_drugs_supplies = icmvu.get_IC_num_drugs_supplies(drug_supply_records)

    #######################################################################################################
    #                                                                                                     #
    #                                       Pre-results / Calculated                                      #
    #                                                                                                     #
    #######################################################################################################

    # To create the database needed to set these up,
    #   1. Comment out the lines below that read from the databases and uncomment the lines that set the MVs to empty lists.
    #   2. Create a projection. Country and years are irrelevant. IH and IC are the only relevant IHT modules.
    #   3. In ICProj, go to finalize_mod_vars, and uncomment the relevant database code.
    #   4. Run calculations.
    #   5. Uncomment the lines below that read from the databases and comment out the lines that set the MVs to empty lists. (reverse of 1)
    #   6. In ICProj, go to finalize_mod_vars, and comment out the relevant database code. (reverse of 3)
    drug_cost_per_avg_case = icrd.IC_set_drug_cost_per_avg_case(interv_records, num_years_outputs, drug_cost_per_avg_case_DB)
    supply_cost_per_avg_case = icrd.IC_set_supply_cost_per_avg_case(interv_records, num_years_outputs, supply_cost_per_avg_case_DB)
    med_personnel_min_per_avg_case_def = icrd.IC_set_med_personnel_min_per_avg_case_def(interv_records, num_years_outputs, med_personnel_min_per_avg_case_DB)
    # drug_cost_per_avg_case = []
    # supply_cost_per_avg_case = []
    # med_personnel_min_per_avg_case_def = []

    #######################################################################################################
    #                                                                                                     #
    #                              ModVars for special modes                                              #
    #                                                                                                     #
    #######################################################################################################

    # Disease-specific costing / TB Costing

    outpat_visit_inpat_day_unit_costs_3d_list = np.full((icc.IC_NUM_VISIT_TYPES, num_costed_deliv_chans, num_prog_areas), 0.0).tolist()

    # Comprehensive vs specific plan (CVS)

    outpat_inpat_costs_2d_list_CVS = np.full((icc.IC_NUM_VISIT_TYPES, num_costed_deliv_chans), 0.0).tolist()

    ###################################################################################################################
    # 
    #                                             For testing calculations
    #
    ###################################################################################################################

    # t1 = gbu.getYearIdx(first_year, first_year)
    # t2 = gbu.getYearIdx(final_year, first_year)

    # for interv_rec in interv_records:
    #     impact_interv = icmvu.get_IC_impact_interv(interv_rec)

    #     for t in acu.GBRange(t1, t2):
    #         if math.isclose(icmvu.get_IC_interv_PIN(interv_rec, t), 0.0):
    #             icmvu.set_IC_interv_PIN(interv_rec, t, 100.0)

    #         if (not impact_interv) and math.isclose(icmvu.get_IC_interv_non_impact_cov(interv_rec, t), 0.0):
    #             icmvu.set_IC_interv_non_impact_cov(interv_rec, t, 100.0)

    # # To test number of services, change the targ pop of Management of opportunistic infections (an HIV non-impact intervention) to total population from DemProj
    # mgmt_opp_inf_interv_rec = icmvu.get_IC_interv_rec(interv_records, icii.IC_IV_MgmtOppInfHIV_AIDS_MST_ID)
    # icmvu.set_IC_interv_targ_pop_mst_ID(mgmt_opp_inf_interv_rec, icc.IC_TP_TOTAL_POPULATION_MST_ID)

    mod_vars_rec = {
        # Inputs / pre-result / calculated
		
        icmvt.IC_VERSION_TAG_V1                                   : 2,
        icmvt.IC_DATA_CHANGED_TAG_V1                              : value_bool_False, 
        icmvt.IC_PROJ_VALID_TAG_V1                                : value_bool_False, 
        icmvt.IC_PROJ_VALID_DATE_TAG_V1                           : value_date,  
        icmvt.IC_SHOW_ALL_YEARS_INTERV_COV_TAG_V1                 : value_bool_False, 
        icmvt.IC_SHOW_ALL_FP_METHODS_INTERV_COV_TAG_V1            : value_bool_False, 
        icmvt.IC_SHOW_ALL_YEARS_INTERV_PIN_TAG_V1                 : value_bool_False, 
        icmvt.IC_ENTER_DET_PMTCT_DATA_TAG_V1                      : value_bool_False, 
        icmvt.IC_ENTER_NET_AGE_DIST_PERC_TAG_V1                   : value_bool_False, 
        icmvt.IC_POST_TREAT_SURV_FOLLOW_UP_VIS_TAG_V1             : value_bool_False, 
        icmvt.IC_MAX_CUSTOM_DRUG_SUPP_MST_ID_TAG_V1               : value_int, 
        icmvt.IC_COV_YEAR_BEF_BASE_YEAR_ITN_TAG_V1                : value_flt,
        icmvt.IC_NET_LIFETIME_ITN_TAG_V1                          : value_flt,
        icmvt.IC_AVG_NET_AGE_ITN_TAG_V1                           : value_flt,
        #icmvt.IC_NUM_PERC_NETS_BY_AGE_TAG_V1                     : array [1..IC_MaxNetAge] of double;
        #icmvt.IC_DRUG_PRICE_ADJUST_TAG_V1                        : array [IC_Transport..IC_Wastage] of double;
        icmvt.IC_INTERV_RECORDS_TAG                               : interv_records,
        icmvt.IC_TREAT_INPUT_DRUG_SUPPLY_RECORDS_TAG              : TI_drug_supply_records,
        icmvt.IC_TREAT_INPUT_MED_PERSONNEL_RECORDS_TAG            : TI_med_personnel_records,
        icmvt.IC_TREAT_INPUT_OUTPAT_INPAT_RECORDS_TAG             : TI_outpat_inpat_records,
        icmvt.IC_TREAT_INPUT_GROUP_DRUG_SUPPLY_RECORDS_TAG        : TIG_drug_supply_records,
        icmvt.IC_TREAT_INPUT_GROUP_MED_PERSONNEL_RECORDS_TAG      : TIG_med_personnel_records,
        icmvt.IC_TREAT_INPUT_GROUP_OUTPAT_INPAT_RECORDS_TAG       : TIG_outpat_inpat_records,
        icmvt.IC_DRUG_SUPPLY_RECORDS_TAG                          : drug_supply_records,
        icmvt.IC_TARG_POP_RECORDS_TAG                          : targ_pop_records,
        #icmvt.IC_NEED_BRST_CNCR_TRT_FLLW_UP_TAG_V1               :
        #icmvt.IC_OWNER_USAGE_RATIO_ARRAY_ITN_TAG_V1              : TDoubleDynArray;
        #icmvt.IC_STRATA_INTERV_LIST_TAG_V1                       :
        #icmvt.IC_ALT_COV_LIST_TAG_V1                             :
		icmvt.IC_ALT_PIN_AGE_GROUP_SET_1_LIST_TAG_V1              : alt_PIN_age_group_set_1_records,
        icmvt.IC_ALT_PIN_AGE_GROUP_SET_2_LIST_TAG_V1              : alt_PIN_age_group_set_2_records,
        #icmvt.IC_STRATUM_RECORDS_TAG_V1                          : array of TObjectList; // mark0310
        #icmvt.IC_OUTP_VISIT_INP_DAY_UNIT_COST_TAG_V1             : GB_TDoubleDyn3DArray;
        #icmvt.IC_NON_DEF_TARG_POPS_TAG_V1                        : array of GB_TIntSet; // mark1207
        icmvt.IC_VARY_TREATMENT_INPUTS_BY_TIME_TAG_V1             : value_bool_True,
        # '<IC_DrugCostPerCase_T1>'                               : np.full((460, 5, 60), 0.0).tolist(),
        # '<IC_SupplyCostPerCase_T1>'                             : np.full((460, 5, 60), 0.0).tolist(),
        # '<IC_LaborCostPerCase_T1>'                              : np.full((460, 5, 60), 0.0).tolist(),
        # '<IC_MinPerAvgCaseCustomStaff_T1>'                      : np.full((460, 5, 40, 60), 0).tolist(),
        # '<IC_MinPerAvgCaseDefaultStaff_T1>'                     : np.full((460, 5, 40, 60), 0).tolist(),
        icmvt.IC_DRUG_COST_PER_CASE_TAG_V1                        : drug_cost_per_avg_case,
        icmvt.IC_SUPPLY_COST_PER_CASE_TAG_V1                      : supply_cost_per_avg_case,
        icmvt.IC_INPATIENT_DAYS_PER_CASE_TAG_V1                   : [],
        icmvt.IC_OUTPATIENT_VISITS_PER_CASE_TAG_V1                : [],
        icmvt.IC_MIN_PER_AVG_CASE_STATIC_STAFF_TAG_V1             : med_personnel_min_per_avg_case_def,
        icmvt.IC_MIN_PER_AVG_CASE_DYNAMIC_STAFF_TAG_V1            : value_list,
        # icmvt.IC_POP_IN_NEED_VISITED_TAG_V1                       : False,
        icmvt.IC_FAM_PLAN_GOAL_TAG_V1                             : icc.CPR_GOAL_MST_ID, #icc.UNMET_NEED_GOAL_MST_ID, 
        icmvt.IC_PERC_UNMET_NEED_COVERED_TAG_V1                   : icmvd.IC_init_year_arr(num_years_inputs, 0.0),

        # Outputs / results

        icmvt.IC_RES_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1           : icmvd.IC_init_interv_deliv_chan_all_year_arr(num_intervs, num_deliv_chan_IC, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_TARG_POPS_TAG_V1                            : icmvd.IC_init_interv_year_arr(num_intervs, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_PEOPLE_NEEDING_SERVICES_TAG_V1              : icmvd.IC_init_interv_year_arr(num_intervs, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_NUM_SERVICES_TAG_V1                         : icmvd.IC_init_interv_year_arr(num_intervs, num_years_outputs, 0.0).tolist(), 
        #icmvt.IC_RES_ADDIT_NUM_SERVICES_BY_DELIV_CHAN_TAG_V1     : [],
        #icmvt.IC_RES_ADDIT_NUM_SERVICES_TAG_V1                   : [],
        icmvt.IC_RES_PERC_DELIV_THROUGH_CHAN_BY_YEAR_TAG_V1      : icmvd.IC_init_interv_deliv_chan_all_year_arr(num_intervs, num_deliv_chan_IC, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DRUG_COSTS_TAG_V1                           : icmvd.IC_init_interv_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        #icmvt.IC_RES_ADDIT_DRUG_COSTS_TAG_V1                     : [],
        icmvt.IC_RES_DRUG_COSTS_BY_YEAR_ONLY_TAG_V1              : icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_SUPPLY_COSTS_TAG_V1                         : icmvd.IC_init_interv_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        #icmvt.IC_RES_ADDIT_SUPPLY_COSTS_TAG_V1                   : [],
        icmvt.IC_RES_SUPPLY_COSTS_BY_YEAR_ONLY_TAG_V1            : icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist(),
        #icmvt.IC_RES_ADDIT_DRUG_SUPPLY_COSTS_TAG_V1              : [],
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1  : icmvd.IC_init_static_staff_costed_deliv_chan_year_arr(num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1      : icmvd.IC_init_static_staff_interv_year_arr(num_intervs, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1   : icmvd.IC_init_static_staff_prog_area_year_arr(num_prog_areas, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_STATIC_STAFF_TIME_REQ_BY_PROG_AREA_DELIV_CHAN_TAG_V1   : icmvd.IC_init_static_staff_prog_area_deliv_chan_year_arr(num_prog_areas, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_DELIV_CHAN_TAG_V1 : icmvd.IC_init_dynamic_staff_costed_deliv_chan_year_arr(num_staff_types, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_INTERV_TAG_V1     : icmvd.IC_init_dynamic_staff_interv_year_arr(num_staff_types, num_intervs, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_TAG_V1  : icmvd.IC_init_dynamic_staff_prog_area_year_arr(num_staff_types, num_prog_areas, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DYNAMIC_STAFF_TIME_REQ_BY_PROG_AREA_DELIV_CHAN_TAG_V1  : icmvd.IC_init_dynamic_staff_prog_area_deliv_chan_year_arr(num_staff_types, num_prog_areas, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        # These two are too large and need to be converted to records.
        #icmvt.IC_RES_DRUG_UNITS_PER_AVG_CASE_TAG_V1              : icmvd.IC_init_drug_supply_interv_deliv_chan_year_arr(num_drugs_supplies, num_intervs, iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS, num_years_outputs, 0.0).tolist(),
        #icmvt.IC_RES_SUPPLY_UNITS_PER_AVG_CASE_TAG_V1            : icmvd.IC_init_drug_supply_interv_deliv_chan_year_arr(num_drugs_supplies, num_intervs, iccl.IC_IH_NUM_DEFAULT_DELIV_CHANS, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_DRUGS_REQUIRED_BY_DELIV_CHAN_TAG            : icmvd.IC_init_drug_supply_costed_deliv_chan_year_arr(num_drugs_supplies, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_SUPPLIES_REQUIRED_BY_DELIV_CHAN_TAG         : icmvd.IC_init_drug_supply_costed_deliv_chan_year_arr(num_drugs_supplies, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
       
        icmvt.IC_RES_DRUGS_REQUIRED_BY_PROG_AREA_TAG            : icmvd.IC_init_drug_supply_prog_area_year_arr(num_drugs_supplies, num_prog_areas, num_years_outputs, 0.0).tolist(),
        icmvt.IC_RES_SUPPLIES_REQUIRED_BY_PROG_AREA_TAG         : icmvd.IC_init_drug_supply_prog_area_year_arr(num_drugs_supplies, num_prog_areas, num_years_outputs, 0.0).tolist(),
       
        icmvt.IC_RES_OUTPAT_VISITS_INPAT_DAYS_TAG_V1             : icmvd.IC_init_interv_visit_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist(),
		
		# Comprehensive vs specific plan (CVS)

        icmvt.IC_OUTPAT_INPAT_COSTS_CVS_TAG_V1                  : outpat_inpat_costs_2d_list_CVS # input
    }

    # Disease-specific costing (DSC), TB Costing

    # If TB costing is on, we'll need access to the IHT interventions well, because the user is allowed to move IHT interventions into TB. 
    if costing_mode == gbc.GB_TB_COSTING_MODE:
        
        #icmvt.IC_OUTPAT_VISIT_INPAT_DAY_UNIT_COSTS_DSC_TAG_V1   : outpat_visit_inpat_day_unit_costs_3d_list, # input
        mod_vars_rec[icmvt.IC_RES_OUTPAT_VISIT_INPAT_DAY_COSTS_DSC_TAG_V1] = icmvd.IC_init_interv_visit_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist()

        mod_vars_rec[icmvt.IC_RES_LOG_SUPPLY_CHAIN_COSTS_TB_TAG] = icmvd.IC_init_drug_supply_year_arr(num_drugs_supplies, num_years_outputs)
        mod_vars_rec[icmvt.IC_RES_WASTAGE_COSTS_TB_TAG] = icmvd.IC_init_drug_supply_year_arr(num_drugs_supplies, num_years_outputs)
        
        # If TB costing is on, we'll need access to the IHT interventions as well, because the user is allowed to move IHT interventions into TB. 
        mod_vars_rec[icmvt.IC_OUTPAT_INPAT_COSTS_CVS_TAG_V1] = outpat_visit_inpat_day_costs_TB
        mod_vars_rec[icmvt.IC_INTERV_RECORDS_IHT_ID_INFO_ONLY_TB_TAG_V1] = interv_records_IHT_ID_info_only

    # LiST Costing

    elif costing_mode == gbc.GB_LIST_COSTING_MODE:

        mod_vars_rec[icmvt.IC_OTHER_RECURRENT_COSTS_CS_TAG_V1] = other_recurrent_costs
        mod_vars_rec[icmvt.IC_CAPITAL_COSTS_CS_TAG_V1] = capital_costs
        mod_vars_rec[icmvt.IC_SUPPLY_CHAIN_COST_PERC_COMMODITY_COST_CS_TAG_V1] = supply_chain_cost_perc_commodity_cost
        mod_vars_rec[icmvt.IC_WASTAGE_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_inputs, 5).tolist()
        mod_vars_rec[icmvt.IC_INFRASTRUCTURE_INVESTMENT_CS_TAG_V1] = infrastructure_investment
        mod_vars_rec[icmvt.IC_INFRASTRUCTURE_ALLOC_PORTION_FOR_PROJ_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_inputs, 0.5).tolist()
        mod_vars_rec[icmvt.IC_RATIO_OTHER_HEALTH_SYSTEM_TO_INTERV_COSTS_CS_TAG_V1] = ratio_other_health_system_to_interv_costs
        mod_vars_rec[icmvt.IC_HEALTH_SYSTEM_PROG_COSTS_SUM_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_inputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_OTHER_HEALTH_SYSTEM_COSTS_VIS_CS_TAG_V1] = False
        mod_vars_rec[icmvt.IC_INCOME_LEVEL_CS_META_TAG_V1] = income_level
        mod_vars_rec[icmvt.IC_SUPPLY_CHAIN_STATUS_CS_META_TAG_V1] = supply_chain_status

        mod_vars_rec[icmvt.IC_RES_OTHER_RECURRENT_COSTS_CS_TAG_V1] = icmvd.IC_init_interv_visit_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_CAPITAL_COSTS_CS_TAG_V1] = icmvd.IC_init_interv_visit_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_PROGRAM_COSTS_CS_TAG_V1] = icmvd.IC_init_costing_sect_year_arr(num_costing_sections_CS, num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_LOGISTICS_COSTS_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_WASTAGE_COSTS_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_INFRASTRUCTURE_INVESTMENT_COSTS_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_RES_OTHER_HEALTH_SYSTEM_COSTS_CS_TAG_V1] = icmvd.IC_init_year_arr(num_years_outputs, 0.0).tolist()

        mod_vars_rec[icmvt.IC_COSTS_BY_INT_CS_TAG_V1] = icmvd.IC_init_interv_arr(num_intervs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_DISC_SOCIAL_BENEFITS_BY_INT_CS_TAG_V1] = icmvd.IC_init_interv_arr(num_intervs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_DISC_ECON_BENEFITS_BY_INT_CS_TAG_V1] = icmvd.IC_init_interv_arr(num_intervs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_DISC_INC_IN_EARN_REDUC_BY_INT_CS_TAG_V1] = icmvd.IC_init_interv_arr(num_intervs, 0.0).tolist()
        mod_vars_rec[icmvt.IC_BCR_BY_INT_CS_TAG_V1] = icmvd.IC_init_interv_arr(num_intervs, 0.0).tolist()
        
    if (
        ((costing_mode == gbc.GB_IHT_COSTING_MODE) and (plan_type == iccl.IC_IH_SPECIFIC_PLAN)) 
        or (costing_mode == gbc.GB_TB_COSTING_MODE) 
        or (costing_mode == gbc.GB_LIST_COSTING_MODE)
    ):
        mod_vars_rec[icmvt.IC_RES_LABOR_COSTS_DSC_TAG_V1] = icmvd.IC_init_interv_costed_deliv_chan_year_arr(num_intervs, num_costed_deliv_chans, num_years_outputs, 0.0).tolist()#icmvd.IC_init_interv_deliv_chan_all_year_arr(num_intervs, num_deliv_chan_IC, num_years_outputs, 0.0).tolist()

    #Debugging
    #with open('C:\Proj\ICSampleFile.JSON', 'w') as f:
    #    json.dump(mod_vars_dict, f)

    return mod_vars_rec